# -*- coding: utf-8 -*-

# Automatically generated - don't edit.
# Use `python setup.py build_ui` to update it.

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formGroupBox = QtWidgets.QGroupBox(Form)
        self.formGroupBox.setObjectName("formGroupBox")
        self.formLayout = QtWidgets.QFormLayout(self.formGroupBox)
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.numOfIterationsLabel = QtWidgets.QLabel(self.formGroupBox)
        self.numOfIterationsLabel.setObjectName("numOfIterationsLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.numOfIterationsLabel)
        self.numOfIterationsSpinBox = QtWidgets.QSpinBox(self.formGroupBox)
        self.numOfIterationsSpinBox.setMaximum(999999999)
        self.numOfIterationsSpinBox.setProperty("value", 300)
        self.numOfIterationsSpinBox.setObjectName("numOfIterationsSpinBox")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.numOfIterationsSpinBox)
        self.toleranceLabel = QtWidgets.QLabel(self.formGroupBox)
        self.toleranceLabel.setObjectName("toleranceLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.toleranceLabel)
        self.toleranceDoubleSpinBox = QtWidgets.QDoubleSpinBox(self.formGroupBox)
        self.toleranceDoubleSpinBox.setDecimals(5)
        self.toleranceDoubleSpinBox.setSingleStep(0.001)
        self.toleranceDoubleSpinBox.setProperty("value", 0.001)
        self.toleranceDoubleSpinBox.setObjectName("toleranceDoubleSpinBox")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.toleranceDoubleSpinBox)
        self.alpha1Label = QtWidgets.QLabel(self.formGroupBox)
        self.alpha1Label.setObjectName("alpha1Label")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.alpha1Label)
        self.alpha1DoubleSpinBox = QtWidgets.QDoubleSpinBox(self.formGroupBox)
        self.alpha1DoubleSpinBox.setDecimals(5)
        self.alpha1DoubleSpinBox.setSingleStep(0.001)
        self.alpha1DoubleSpinBox.setProperty("value", 0.001)
        self.alpha1DoubleSpinBox.setObjectName("alpha1DoubleSpinBox")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.alpha1DoubleSpinBox)
        self.alpha2Label = QtWidgets.QLabel(self.formGroupBox)
        self.alpha2Label.setObjectName("alpha2Label")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.alpha2Label)
        self.alpha2DoubleSpinBox = QtWidgets.QDoubleSpinBox(self.formGroupBox)
        self.alpha2DoubleSpinBox.setDecimals(8)
        self.alpha2DoubleSpinBox.setSingleStep(1e-06)
        self.alpha2DoubleSpinBox.setProperty("value", 1e-06)
        self.alpha2DoubleSpinBox.setObjectName("alpha2DoubleSpinBox")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.alpha2DoubleSpinBox)
        self.lambdaLabel = QtWidgets.QLabel(self.formGroupBox)
        self.lambdaLabel.setObjectName("lambdaLabel")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.lambdaLabel)
        self.lambdaDoubleSpinBox = QtWidgets.QDoubleSpinBox(self.formGroupBox)
        self.lambdaDoubleSpinBox.setDecimals(8)
        self.lambdaDoubleSpinBox.setSingleStep(1e-06)
        self.lambdaDoubleSpinBox.setProperty("value", 1e-06)
        self.lambdaDoubleSpinBox.setObjectName("lambdaDoubleSpinBox")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.lambdaDoubleSpinBox)
        self.computerScoreLabel = QtWidgets.QLabel(self.formGroupBox)
        self.computerScoreLabel.setObjectName("computerScoreLabel")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.computerScoreLabel)
        self.computerScoreCheckBox = QtWidgets.QCheckBox(self.formGroupBox)
        self.computerScoreCheckBox.setObjectName("computerScoreCheckBox")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.computerScoreCheckBox)
        self.fitInterceptLabel = QtWidgets.QLabel(self.formGroupBox)
        self.fitInterceptLabel.setObjectName("fitInterceptLabel")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.fitInterceptLabel)
        self.fitInterceptCheckBox = QtWidgets.QCheckBox(self.formGroupBox)
        self.fitInterceptCheckBox.setChecked(True)
        self.fitInterceptCheckBox.setObjectName("fitInterceptCheckBox")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.fitInterceptCheckBox)
        self.normalizeLabel = QtWidgets.QLabel(self.formGroupBox)
        self.normalizeLabel.setObjectName("normalizeLabel")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.normalizeLabel)
        self.normalizeCheckBox = QtWidgets.QCheckBox(self.formGroupBox)
        self.normalizeCheckBox.setObjectName("normalizeCheckBox")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.FieldRole, self.normalizeCheckBox)
        self.verticalLayout.addWidget(self.formGroupBox)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(("Form"))
        self.formGroupBox.setTitle(("Bayesian Ridge"))
        self.numOfIterationsLabel.setText(("num of iterations"))
        self.numOfIterationsSpinBox.setToolTip(("Maximum number of iterations. Default is 300."))
        self.numOfIterationsSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.BayesianRidge.html"))
        self.toleranceLabel.setText(("Tolerance"))
        self.toleranceDoubleSpinBox.setToolTip(("Stop the algorithm if w has converged. Default is 1.e-3."))
        self.toleranceDoubleSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.BayesianRidge.html"))
        self.alpha1Label.setText(("Alpha 1"))
        self.alpha1DoubleSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.BayesianRidge.html"))
        self.alpha2Label.setText(("Alpha 2"))
        self.alpha2DoubleSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.BayesianRidge.html"))
        self.lambdaLabel.setText(("Lambda"))
        self.lambdaDoubleSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.BayesianRidge.html"))
        self.computerScoreLabel.setText(("Compute Score"))
        self.computerScoreCheckBox.setToolTip(_translate("Form", "If True, compute the objective function at each step of the model.\n"
"Default is False"))
        self.computerScoreCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.BayesianRidge.html"))
        self.computerScoreCheckBox.setText(("compute the objective function at each step of the model"))
        self.fitInterceptLabel.setText(("Fit Intercept"))
        self.fitInterceptCheckBox.setToolTip(_translate("Form", "whether to calculate the intercept for this model. If set to false,\n"
"no intercept will be used in calculations (e.g. data is expected to\n"
"be already centered). Default is True."))
        self.fitInterceptCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.BayesianRidge.html"))
        self.fitInterceptCheckBox.setText(("calculate the intercept for this model"))
        self.normalizeLabel.setText(("Normalize"))
        self.normalizeCheckBox.setToolTip(_translate("Form", "This parameter is ignored when fit_intercept is set to False. If True,\n"
"the regressors X will be normalized before regression by subtracting\n"
"the mean and dividing by the l2-norm. If you wish to standardize,\n"
"please use sklearn.preprocessing.StandardScaler before calling fit on\n"
"an estimator with normalize=False."))
        self.normalizeCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.BayesianRidge.html"))
        self.normalizeCheckBox.setText(("regressors X will be normlized before regression"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())




from PyQt5 import QtWidgets
from point_spectra_gui.ui.BayesianRidge import Ui_Form
from point_spectra_gui.util.Modules import Modules
from sklearn.linear_model import BayesianRidge


class Ui_Form(Ui_Form, BayesianRidge, Modules):
    def setupUi(self, Form):
        super().setupUi(Form)
        self.checkMinAndMax()
        self.connectWidgets()

    def get_widget(self):
        return self.formGroupBox

    def setHidden(self, bool):
        self.get_widget().setHidden(bool)

    def connectWidgets(self):
        self.numOfIterationsSpinBox.setValue(self.n_iter)
        self.toleranceDoubleSpinBox.setValue(self.tol)
        self.alpha1DoubleSpinBox.setValue(self.alpha_1)
        self.alpha2DoubleSpinBox.setValue(self.alpha_2)
        self.lambdaDoubleSpinBox.setValue(self.lambda_1)
        self.lambdaDoubleSpinBox.setValue(self.lambda_2)
        self.computerScoreCheckBox.setChecked(self.compute_score)
        self.fitInterceptCheckBox.setChecked(self.fit_intercept)
        self.normalizeCheckBox.setChecked(self.normalize)

    def run(self):
        params = {'n_iter': self.numOfIterationsSpinBox.value(),
                  'tol': self.toleranceDoubleSpinBox.value(),
                  'alpha_1': self.alpha1DoubleSpinBox.value(),
                  'alpha_2': self.alpha2DoubleSpinBox.value(),
                  'lambda_1': self.lambdaDoubleSpinBox.value(),
                  'lambda_2': self.lambdaDoubleSpinBox.value(),
                  'compute_score': self.computerScoreCheckBox.isChecked(),
                  'fit_intercept': self.fitInterceptCheckBox.isChecked(),
                  'normalize': self.normalizeCheckBox.isChecked(),
                  }
        return params, self.getChangedValues(params, BayesianRidge())


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



from PyQt5 import QtWidgets, QtCore
from sklearn.gaussian_process.gaussian_process import GaussianProcess

from point_spectra_gui.ui.cv_GP import Ui_Form
from point_spectra_gui.util.Modules import Modules


class Ui_Form(Ui_Form, GaussianProcess, Modules):
    def setupUi(self, Form):
        super().setupUi(Form)
        self.connectWidgets()
        self.checkMinAndMax()

    def get_widget(self):
        return self.formGroupBox

    def setHidden(self, bool):
        self.get_widget().setHidden(bool)

    def connectWidgets(self):
        self.DimRedList.setCurrentItem(self.DimRedList.findItems('PCA', QtCore.Qt.MatchExactly)[0])
        self.regression_list.setCurrentItem(self.regression_list.findItems('Linear', QtCore.Qt.MatchExactly)[0])
        self.CorrelationList.setCurrentItem(
            self.CorrelationList.findItems('Squared Exponential', QtCore.Qt.MatchExactly)[0])
        self.theta0LineEdit.setText(str(self.theta0))
        self.ThetaL.setText(str(self.thetaL))
        self.ThetaU.setText(str(self.thetaU))
        self.randomStartLineEdit.setText(str(self.random_start))
        self.normalize_list.setCurrentItem(
            self.normalize_list.findItems(str(self.normalize), QtCore.Qt.MatchExactly)[0])

    def run(self):
        normalize_items = [i.text() == 'True' for i in self.normalize_list.selectedItems()]
        regr_items = [str(i.text().lower()) for i in self.regression_list.selectedItems()]
        corr_items = [str(i.text().lower().replace(' ', '_')) for i in self.CorrelationList.selectedItems()]
        dim_red_items = [str(i.text()) for i in self.DimRedList.selectedItems()]
        params = {
            'reduce_dim': dim_red_items,
            'n_components': [int(i) for i in self.numOfComponenetsLineEdit.text().split(',')],
            'regr': regr_items,
            'corr': corr_items,
            'storage_mode': ['light'],
            'verbose': [True],
            'theta0': [float(i) for i in self.theta0LineEdit.text().split(',')],
            # where are the bounds on Theta? need to check this
            'normalize': [normalize_items],
            'optimizer': ['fmin_cobyla'],
            'random_start': [int(i) for i in self.randomStartLineEdit.text().split(',')],
        }

        modelkey = str(params)
        return params, modelkey


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



import multiprocessing as mp
import os.path
import pickle
import sys
import time
import warnings
import json

from point_spectra_gui.util.themes import braceyourself, default

try:
    import qtmodern.styles

    q = True
except:
    q = False
    warnings.warn("You're missing the qtmodern package."
                  "to install it use pip install qtmodern")
import os
import logging
import platform
import traceback
from time import strftime

from PyQt5.QtCore import *
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import *
from PyQt5 import QtCore, QtWidgets, QtGui
from PyQt5.QtCore import QSettings

from point_spectra_gui import core, __version__
from point_spectra_gui.ui.MainWindow import Ui_MainWindow
from point_spectra_gui.util import delete
from point_spectra_gui.util.Modules import Modules
from point_spectra_gui.util.excepthook import my_exception_hook


class EmittingStream(QtCore.QObject):
    textWritten = QtCore.pyqtSignal(str)

    def write(self, text):
        self.textWritten.emit(str(text))

    def flush(self):
        pass


class TitleWindow:
    """
    Top portion of the application needs a name.
    Displays the name of our restored, or saved file
    Displays whether we are debugging or not
    """

    def __init__(self, mainName):
        self.mainName = mainName
        self.fileName = ''
        self.debugName = ''

    def setMainName(self, name):
        self.mainName = name

    def setFileName(self, name):
        self.fileName = name

    def setDebugName(self, bool):
        if bool:
            self.debugName = "Debug Mode"
        else:
            self.debugName = ''

    def display(self):
        if self.fileName == '' and self.debugName == '':
            return "{}".format(self.mainName)
        elif self.fileName == '':
            return "{} - {}".format(self.mainName, self.debugName)
        elif self.debugName == '':
            return "{} - {}".format(self.mainName, self.fileName)
        else:
            return "{} - {} - {}".format(self.mainName, self.fileName, self.debugName)


class MainWindow(Ui_MainWindow, QtCore.QThread, Modules):
    """
    The Main part of the application where everything magical happens
    """
    taskFinished = QtCore.pyqtSignal()

    def __init__(self):
        super().__init__()
        self.widgetList = []
        self.leftOff = 0
        Modules.parent = [self]

    def setupUi(self, MainWindow):
        super().setupUi(MainWindow)  # Run the basic window UI
        self.MainWindow = MainWindow
        self.MainWindow.closeEvent = self.closeEvent
        self.title = TitleWindow(self.MainWindow.windowTitle())
        self._readAndApplyWindowAttributeSettings()
        self.menu_item_shortcuts()  # set up the shortcuts
        self.connectWidgets()

        # Check the mode for debugging
        if self.settings.value("debug") == 'true':
            self.debug_mode()
        else:
            self.normal_mode()

        # Check the theme for the UI
        if self.settings.value('theme') == 'braceyourself':
            braceyourself(self.MainWindow)

    def normal_mode(self):
        """
        Change the direction of stdout to print to the UI console instead

        :return:
        """
        sys.stdout = EmittingStream(textWritten=self.normalOutputWritten)
        sys.stderr = sys.__stderr__
        # sys.stderr = EmittingStream(textWritten=self.normalOutputWritten)
        self._mode(self.actionOn, self.actionOff, False)

    def debug_mode(self):
        """
        Change the direction of stdout to print to the original console

        :return:
        """
        sys.stdout = EmittingStream(textWritten=self.normalOutputWritten)
        sys.stderr = sys.__stderr__
        self._mode(self.actionOff, self.actionOn, True)

    def _mode(self, obj1, obj2, debug):
        """
        Set the UI in debug or non-debug mode
        Save all the settings and grey out the necessary boxes

        :param obj1:
        :param obj2:
        :param debug:
        :return:
        """
        obj1.setDisabled(False)
        obj2.setDisabled(True)
        self.debug = debug
        self.title.setDebugName(debug)
        self.settings.setValue("debug", self.debug)
        self.MainWindow.setWindowTitle(self.title.display())

    def theme(self, name):
        """
        We have 3 themes
        each has a different situation
                   __________________
                 /__               __\ new()   Something to note:
               v    \            v    \ new()  As you can see whenever moving into or out of
        default     braceyourself    qtmodern  qtmodern we will have to start a new session
               \___^             \___^ new()   default and braceyourself can simply change
                \___________________/ new()    there styling on the spot

        :param name:
        :return:
        """
        settings = self.settings.value('theme')
        if name == settings:
            print("This is already your current theme")

        elif name == 'qtmodern':  # User is entering into qtmodern
            self.settings.setValue('theme', name)
            self.new()

        elif settings == 'qtmodern':  # User is leaving qtmodern
            self.settings.setValue('theme', name)
            self.new()

        elif name == 'default':
            self.settings.setValue('theme', name)
            default(self.MainWindow)

        elif name == 'braceyourself':
            self.settings.setValue('theme', name)
            braceyourself(self.MainWindow)
        else:
            print("Something went horribly wrong with your theme, try again?")

    def normalOutputWritten(self, text):
        """Append text to the QTextEdit."""
        # Maybe QTextEdit.append() works as well, but this is how I do it:
        cursor = self.textBrowser.textCursor()
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertText(text)
        self.textBrowser.setTextCursor(cursor)
        self.textBrowser.ensureCursorVisible()

    def addWidget(self, obj):
        """
        Organize our widgets using a list
        Each widget is addressed separately due to being in a list

        :param obj:
        :return:
        """
        # add the new object to our widgetList
        # run setupUi method
        # creating a new layout
        # giving that layout a name
        # add the new layout to verticalLayout_3
        # add our object's widget to the layout
        # scroll down to the new module
        # TODO this needs to be done a little bit better. The UI should know if we are on "Insert Module" or not and then decide from there.
        try:
            idx = self.insert_idx
            if idx == 0:
                raise Exception
            self.widgetList.insert(idx, obj())
        except:
            idx = -1
            self.widgetList.append(obj())
        self.widgetList[idx].setupUi(self.centralwidget)
        self.widgetLayout = QtWidgets.QVBoxLayout()
        self.widgetLayout.setObjectName("widgetLayout")
        self.verticalLayout_2.insertLayout(idx, self.widgetLayout)
        self.widgetLayout.addWidget(self.widgetList[idx].get_widget())
        scrollbar = self.scrollArea.verticalScrollBar()
        scrollbar.setValue(scrollbar.maximum())
        # place items inside the deleteModuleCombox
        # This populates the comboBox
        self.deleteModuleComboBox.disconnect()
        self.setComboBox(self.deleteModuleComboBox,
                         ["Delete"] + [type(item).__name__ for item in self.widgetList])
        self.deleteModuleComboBox.currentIndexChanged.connect(self.on_deleteModuleComboBox_changed)
        self.setComboBox(self.insertModuleComboBox,
                         ["Insert After"] + [type(item).__name__ for item in self.widgetList])
        # When the user selects an item in the insertion comboBox we want the UI to insert every time after that selected item
        # When the user deletes a module, we want to update insertion comboBox and deletion comboBox
        # When we insert we want to always keep the index, but update insertion and deletion comboBoxes

    def menu_item_shortcuts(self):
        self.actionExit.setShortcut("ctrl+Q")
        self.actionClear_Workflow.setShortcut("ctrl+K")
        self.actionCreate_New_Workflow.setShortcut("ctrl+N")
        self.actionRestore_Workflow.setShortcut("ctrl+O")
        self.actionSave_Current_Workflow.setShortcut("ctrl+S")
        self.okPushButton.setShortcut("Ctrl+Return")
        self.refreshModulePushButton.setShortcut("Alt+Return")

    def connectWidgets(self):
        """
        Connect all the widgets associated with the MainWindow UI

        :return:
        """
        try:
            self.actionRead_ChemCam_Data.triggered.connect(
                lambda: self.addWidget(core.ReadChemCamData.ReadChemCamData))
            self.actionRemove_Baseline.triggered.connect(
                lambda: self.addWidget(core.BaselineRemoval.BaselineRemoval))
            self.actionCross_Validation.triggered.connect(
                lambda: self.addWidget(core.CrossValidation.CrossValidation))
            self.actionDimensionality_Reduction.triggered.connect(
                lambda: self.addWidget(core.DimensionalityReduction.DimensionalityReduction))
            self.actionCluster.triggered.connect(
                lambda: self.addWidget(core.Clustering.Clustering))
            self.actionInterpolate.triggered.connect(
                lambda: self.addWidget(core.Interpolation.Interpolation))
            self.actionLoad_Data.triggered.connect(
                lambda: self.addWidget(core.LoadData.LoadData))
            self.actionSave_Current_Data.triggered.connect(
                lambda: self.addWidget(core.WriteToCSV.WriteToCSV))
            self.actionRename_Data.triggered.connect(
                lambda: self.addWidget(core.RenameData.RenameData))
            self.actionApply_Mask.triggered.connect(
                lambda: self.addWidget(core.MaskData.MaskData))
            self.actionMultiply_by_Vector.triggered.connect(
                lambda: self.addWidget(core.MultiplyByVector.MultiplyByVector))
            self.actionNormalization.triggered.connect(
                lambda: self.addWidget(core.Normalization.Normalization))
            self.actionSet_Output_Path.triggered.connect(
                lambda: self.addWidget(core.OutputFolder.OutputFolder))
            self.actionPeak_Areas.triggered.connect(
                lambda: self.addWidget(core.PeakAreas.PeakAreas))
            self.actionPlot.triggered.connect(
                lambda: self.addWidget(core.Plot.Plot))
            self.actionPlot_ICA_PCA.triggered.connect(
                lambda: self.addWidget(core.Plot_ICA_PCA.Plot_ICA_PCA))
            self.actionPlot_Spectra.triggered.connect(
                lambda: self.addWidget(core.PlotSpectra.PlotSpectra))
            self.actionTrain.triggered.connect(
                lambda: self.addWidget(core.RegressionTrain.RegressionTrain))
            self.actionPredict.triggered.connect(
                lambda: self.addWidget(core.RegressionPredict.RegressionPredict))
            self.actionRemove_Rows.triggered.connect(
                lambda: self.addWidget(core.RemoveRows.RemoveRows))
            self.actionSplit_Data.triggered.connect(
                lambda: self.addWidget(core.SplitDataset.SplitDataset))
            self.actionOutlier_Removal.triggered.connect(
                lambda: self.addWidget(core.OutlierRemoval.OutlierRemoval))
            self.actionStratified_Folds.triggered.connect(
                lambda: self.addWidget(core.StratifiedFolds.StratifiedFolds))
            self.actionSubmodel_Blend.triggered.connect(
                lambda: self.addWidget(core.SubmodelBlend.SubmodelBlend))
            self.actionSpectral_Derivative.triggered.connect(
                lambda: self.addWidget(core.SpecDeriv.SpecDeriv))
            self.actionCombine_Data_Sets.triggered.connect(
                lambda: self.addWidget(core.CombineDataSets.CombineDataSets))
            self.actionRestore_Trained_Model.triggered.connect(
                lambda: self.addWidget(core.RestoreTrainedModel.RestoreTrainedModel))
            self.actionSave_Trained_Model.triggered.connect(
                lambda: self.addWidget(core.SaveTrainedModel.SaveTrainedModel))
            self.actionData_Box.triggered.connect(self.on_DataTable_clicked)
            self.actionAbout.triggered.connect(self.on_About_clicked)
            self.actionAbout_Qt.triggered.connect(self.on_AboutQt_clicked)
            self.actionQtmodern.triggered.connect(lambda: self.theme('qtmodern'))
            self.actionDefault.triggered.connect(lambda: self.theme('default'))
            self.actionBrace_yourself.triggered.connect(lambda: self.theme('braceyourself'))
            self.actionCreate_New_Workflow.triggered.connect(self.new)
            self.actionClear_Workflow.triggered.connect(self.clear)
            self.actionSave_Current_Workflow.triggered.connect(self.on_save_clicked)
            self.actionRestore_Workflow.triggered.connect(self.on_restore_clicked)
            self.deleteModuleComboBox.currentIndexChanged.connect(self.on_deleteModuleComboBox_changed)
            self.insertModuleComboBox.currentIndexChanged.connect(self.on_insertModuleComboBox_changed)
            self.okPushButton.clicked.connect(self.on_okButton_clicked)
            self.undoModulePushButton.clicked.connect(self.on_Rerun_Button_clicked)
            self.refreshModulePushButton.clicked.connect(self.Refresh_Modules_clicked)
            self.stopPushButton.clicked.connect(self.on_stopButton_clicked)
            self.actionOn.triggered.connect(self.debug_mode)
            self.actionOff.triggered.connect(self.normal_mode)
            self.actionExit.triggered.connect(self.MainWindow.close)
            self.actionSupervised.setEnabled(False)

        except Exception as e:
            print(e)

    def closeEvent(self, event):
        """
        Write window size and position to config file, or system registry
        """
        self._writeWindowAttributeSettings()
        event.accept()

    def getWidgetItems(self):
        """
        This function iterates through widgetList
        gets the name of all the Modules
        and then all of the parameters in the UI
        and write it to a list to be returned

        Iterate through widgetList
        get the names of core, add it to a temp l
        add f list to be the first item in ui_items
        """
        f = []
        ui_items = []
        for f_items in self.widgetList:
            f.append(type(f_items).__name__)
        ui_items.append(f)

        for dat in self.widgetList:
            ui_items.append(dat.getGuiParams())
        return ui_items

    def setWidgetItems(self, dict):
        """
        This function iterates through a dictionary supplied by the parameter `dict`
        The [0]th element of the dictionary is the header that contains the names of each
        module. We use the names as a way to identify what module we want to add to the UI
        Once the full UI has been restored, the parameters are filled in for each individual
        module.

        :param dict:
        :return:
        """
        for f_items in dict[0]:
            """
            Really complex way of running essentially this:
            `self.addWidget(core.SplitDataset.SplitDataset))`
            Part of the reason why we're doing this is because we're saving class
            names to a list, you can't save class instances. So this is the next
            best thing.
            """
            self.addWidget(getattr(getattr(core, f_items), f_items))

        for i in range(1, len(dict)):
            self.widgetList[i - 1].setGuiParams(dict[i])

    def on_save_clicked(self):
        """
        Save the workflow to a *.wrf file

        :return:
        """
        try:
            filename, _filter = QtWidgets.QFileDialog.getSaveFileName(None,
                                                                      "Choose where you want save your file",
                                                                      self.outpath,
                                                                      '(*.json)')
            print(filename)
            with open(filename, 'w') as fp:
                json.dump(self.getWidgetItems(), fp, indent=4)
            self.title.setFileName(filename.split('/')[-1])
            self.MainWindow.setWindowTitle(self.title.display())
        except Exception as e:
            print("Save file not loaded: {}".format(e))

    def on_restore_clicked(self):
        """
        Choose a file
        Set the file to be read
        Pickle load our data and push it through the function setWidgetItems

        :return:
        """
        try:
            self.restorefilename, _filter = QtWidgets.QFileDialog.getOpenFileName(None,
                                                                                  "Open Workflow File",
                                                                                  self.outpath,
                                                                                  '(*.json)')
            print(self.restorefilename)
            with open(self.restorefilename, 'r') as fp:
                self.setWidgetItems(json.load(fp))
            self.title.setFileName(self.restorefilename.split('/')[-1])
            self.MainWindow.setWindowTitle(self.title.display())
        except Exception as e:
            print("Restore file not loaded: {}".format(e))

    def delete_module(self, _idx=1):
        """
        Check to see if the last item is enabled
        If it is, delete the last item in the list
        And then call the del_layout function, which will remove the item from the UI

        :return:
        """

        def idx(total, position):
            return total - position + 1

        try:
            if self.widgetList[_idx - 1].isEnabled():
                self.widgetList[_idx - 1].delete()  # Call internal module's delete method
                del self.widgetList[_idx - 1]  # remove the widget from the list
                delete.del_layout(self.verticalLayout_2, idx(self.verticalLayout_2.count(), _idx))  # delete the layout
            else:
                print("Cannot delete")
        except Exception as e:
            print("Cannot delete: ", e)

    def on_insertModuleComboBox_changed(self):
        self.insert_idx = self.insertModuleComboBox.currentIndex()

    def on_deleteModuleComboBox_changed(self):
        """
        Using a comboBox, a user can delete a particular module
        out of the UI

        :return:
        """

        _idx = self.deleteModuleComboBox.currentIndex()
        _text = self.deleteModuleComboBox.currentText()

        # If the currentText in the comboBox is not '', and 'Delete Module', and the Module is Enabled
        if _text != 'Delete Module' and _text != '' and self.widgetList[_idx - 1].isEnabled():
            self.delete_module(_idx)
            self.deleteModuleComboBox.disconnect()
            self.deleteModuleComboBox.removeItem(_idx)
            self.insertModuleComboBox.removeItem(_idx)
            self.deleteModuleComboBox.setCurrentIndex(0)
            self.deleteModuleComboBox.currentIndexChanged.connect(self.on_deleteModuleComboBox_changed)
        else:
            self.deleteModuleComboBox.disconnect()
            self.deleteModuleComboBox.setCurrentIndex(0)
            self.deleteModuleComboBox.currentIndexChanged.connect(self.on_deleteModuleComboBox_changed)
            print("Cannot delete")

    def on_okButton_clicked(self):
        """
        Start the multithreading function

        :return:
        """
        self.onStart()
        self.taskFinished.connect(self.onFinished)

    def on_Rerun_Button_clicked(self):
        """
        Check to see if the last item in the list is enabled
        subtract 1 from leftOff
        enable the current leftOff item

        :return:
        """
        try:
            if not self.widgetList[self.leftOff - 1].isEnabled():
                if self.leftOff > 0:
                    self.leftOff -= 1
                self.widgetList[self.leftOff].setDisabled(False)
            else:
                print("Cannot Re-run modules")
        except:
            pass

    def Refresh_Modules_clicked(self):
        self.setupModules()

    def on_stopButton_clicked(self):
        """
        Hard terminate running thread

        Technically you should never do this.
        But given that some tasks are monumentally huge,
        I feel that it is justified.

        :return:
        """
        if self.isRunning():
            self.terminate()
            self.taskFinished.emit()
        else:
            print("There is nothing running right now")

    def on_AboutQt_clicked(self):
        self.aboutqt = core.AboutQt.AboutQT()
        self.aboutqt.show()

    def on_About_clicked(self):
        """
        Display an about window that lets the user know what version their application is

        :return:
        """
        self.aboutForm = core.About.About()
        self.aboutForm.show()

    def on_DataTable_clicked(self):
        self.dataForm = core.DataTable.DataTable()
        self.dataForm.show()

    def _writeWindowAttributeSettings(self):
        '''
        Save window attributes as settings.

        Called when window moved, resized, or closed.
        '''
        self.settings.beginGroup("mainWindow")
        self.settings.setValue("pos", self.MainWindow.pos())
        self.settings.setValue("maximized", self.MainWindow.isMaximized())
        if not self.MainWindow.isMaximized():
            self.settings.setValue("size", self.MainWindow.size())

        self.settings.endGroup()

    def _readAndApplyWindowAttributeSettings(self):
        '''
        Read window attributes from settings,
        using current attributes as defaults (if settings not exist.)

        Called at QMainWindow initialization, before show().
        '''
        self.settings.beginGroup("mainWindow")
        # No need for toPoint, etc. : PySide converts types
        try:
            self.MainWindow.move(self.settings.value("pos"))
            if self.settings.value("maximized") in 'true':
                self.MainWindow.showMaximized()
            else:
                self.MainWindow.resize(self.settings.value("size"))
        except:
            pass
        self.settings.endGroup()

    def onStart(self):
        """
        This is multithreading thus run() == start()
        make the bar pulse green
        TaskThread.start()

        :return:
        """
        self.progressBar.setRange(0, 0)
        self.start()

    def onFinished(self):
        """
        When a given task is finished
        stop the bar pulsing green
        and display 100% for bar.

        :return:
        """
        self.progressBar.setRange(0, 1)
        self.progressBar.setValue(1)

    def clear(self):
        """
        Delete all modules in GUI
        Except those that are greyed out

        :return:
        """
        while len(self.widgetList) > 0 and self.widgetList[-1].isEnabled():
            self.delete_module()
        self.title.setFileName('')
        self.MainWindow.setWindowTitle(self.title.display())
        self.textBrowser.clear()

    def new(self):
        """
        Start a new gui

        :return:
        """
        self._writeWindowAttributeSettings()
        p = mp.Process(target=main, args=())
        p.start()

    def setupModules(self):
        """
        This function iterates through a list of object addresses
        which then run its dot notated setup()

        iterate through our widgets, start from the last left off item
        run our current module's setup()

        :return:
        """
        dic = self.getWidgetItems()
        for modules in range(self.leftOff, len(self.widgetList)):
            if self.debug:
                self.widgetList[modules].setup()
            else:
                self._logger(self.widgetList[modules].setup)
            self.widgetList[modules].disconnectWidgets()
            self.widgetList[modules].connectWidgets()
            self.widgetList[modules].selectiveSetGuiParams(dic[modules + 1])

    def runModules(self):
        """
        This function iterates through a list of object addresses
        which then run its dot notated run()

        iterate through our widgets, start from the last left off item
        get the name of our current widget item
        start the timer
        print the name of the module running
        run our current module's function()
        get our end time
        print how long it took our current module to execute based on start time and end time
        disable our current module
        increment our left off module

        :return:
        """

        for modules in range(self.leftOff, len(self.widgetList)):
            name_ = type(self.widgetList[modules]).__name__
            s = time.time()
            print("{} Module is Running...".format(name_))
            self.widgetList[modules].run()
            e = time.time()
            print("Module {} executed in: {} seconds".format(name_, e - s))
            self.widgetList[modules].setDisabled(True)
            self.leftOff = modules + 1

    def _logger(self, function):
        try:
            function()
        except Exception as e:
            print("Your {} module broke with error: {}.".format(type(self.widgetList[self.leftOff]).__name__, e))
            self.widgetList[self.leftOff].setDisabled(False)

    def _exceptionLogger(self, function):
        """
        Logs an exception that occurs during the running of a function
        Take a function address in as an input

        :param function:
        :return:
        """
        logfile = "file.log"
        logpath = os.path.join(os.path.join(os.environ['USERPROFILE']), 'Desktop\\logs')

        try:
            function()
        except:
            if not os.path.exists(logpath):
                os.makedirs(logpath)
            timenow = strftime('%d-%m-%y_%H-%M-%S')
            logfilename = "%s_%s" % (timenow, logfile)
            filename = os.path.join(str(os.getcwd()), "%s" % (os.path.join(logpath, logfilename)))
            logging.basicConfig(level=logging.DEBUG, filename=filename)
            logging.exception(
                '[%s %s] (%s) Version: %s:' % (platform.system(), platform.release(), timenow, __version__))
            traceback.print_exc()
            print('\nException was logged to "%s"' % (os.path.join(logpath, logfilename)))

    def run(self):
        """
        Start the thread for running all the modules

        :return:
        """
        if self.debug:
            self._exceptionLogger(self.runModules)
        else:
            self._logger(self.runModules)
        self.taskFinished.emit()


def get_splash(app):
    """
    Get the splash screen for the application
    But check to see if the image even exists

    :param app:
    :return:
    """
    dirs = ['../images/', '/point_spectra_gui/images', './point_spectra_gui/images']
    for dir in dirs:
        if os.path.exists(dir + 'splash.png'):
            splash_pix = QPixmap(dir + 'splash.png')  # default
            app_icon = QtGui.QIcon(dir + 'icon.png')
            app.setWindowIcon(app_icon)
            splash = QSplashScreen(splash_pix, Qt.WindowStaysOnTopHint)
            splash.setMask(splash_pix.mask())
            splash.show()
            time.sleep(0.5)
            app.processEvents()
            return 0


def setDarkmode(app):
    """
    Start the darkmode for the application
    """
    settings = QSettings('USGS', 'PPSG')
    p = settings.value('theme') == 'qtmodern'
    if q and p:
        qtmodern.styles.dark(app)


def main():
    sys._excepthook = sys.excepthook
    sys.excepthook = my_exception_hook
    app = QtWidgets.QApplication(sys.argv)
    get_splash(app)
    setDarkmode(app)
    mainWindow = QtWidgets.QMainWindow()
    ui = MainWindow()
    ui.setupUi(mainWindow)
    mainWindow.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
Created on Fri Dec  4 14:53:23 2015

@author: rbanderson
"""
import pandas as pd

class spectral_data(object):
    def __init__(self, df, dim_red = None):

        try:
            uppercols = df.columns.levels[0]
            lowercols = list(df.columns.levels[1].values)
        except:
            df.columns = pd.MultiIndex.from_tuples(list(df.columns))
            uppercols = df.columns.levels[0]
            lowercols = list(df.columns.levels[1].values)

        for i, val in enumerate(lowercols):
            try:
                lowercols[i] = float(val)
            except:
                lowercols[i] = val

        levels = [uppercols, lowercols]
        df.columns.set_levels(levels, inplace=True)
        self.df = df
        if dim_red is not None:
            self.dim_red = dim_red  #this is a temporary fix to keep track of dimensionality reduction loadings for plotting purposes.
                                    #TODO: Make this robust to other transforms being applied to the data. Currently, many transforms will strip this off when they re-define the spectral data object






from PyQt5 import QtWidgets

from point_spectra_gui.util import Qtickle
from point_spectra_gui.core.outlierRemovalMethods import *
from point_spectra_gui.ui.OutlierRemoval import Ui_Form
from point_spectra_gui.util.Modules import Modules
from libpysat.utils.outlier_removal import outlier_removal
from point_spectra_gui.util.spectral_data import spectral_data

class OutlierRemoval(Ui_Form, Modules):
    def setupUi(self, Form):
        self.Form = Form
        super().setupUi(Form)
        Modules.setupUi(self, Form)
        self.outlierRemovalMethods()

    def get_widget(self):
        return self.groupLayout

    def make_outlier_widget(self, alg, params=None):
        self.hideAll()
        print(alg)
        for i in range(len(self.algorithm_list)):
            if alg == self.algorithm_list[i]:
                self.alg[i - 1].setHidden(False)

    def connectWidgets(self):
        self.algorithm_list = ['Choose an algorithm',
                               'Isolation Forest',
                               'Local Outlier Factor'
                               ]

        self.setComboBox(self.chooseDataComboBox, self.datakeys)
        self.setComboBox(self.chooseAlgorithmComboBox, self.algorithm_list)
        self.changeComboListVars(self.xVariableList, self.xvar_choices())
        self.xvar_choices()
        self.chooseAlgorithmComboBox.currentIndexChanged.connect(
            lambda: self.make_outlier_widget(self.chooseAlgorithmComboBox.currentText()))
        self.chooseDataComboBox.currentIndexChanged.connect(
            lambda: self.changeComboListVars(self.xVariableList, self.xvar_choices()))

    def getGuiParams(self):
        """
        Overriding Modules' getGuiParams, because I'll need to do a list of lists
        in order to obtain the regressionMethods' parameters
        """
        self.qt = Qtickle.Qtickle(self)
        s = []
        s.append(self.qt.guiSave())
        for items in self.alg:
            s.append(items.getGuiParams())
        return s

    def setGuiParams(self, dict):
        self.qt = Qtickle.Qtickle(self)
        self.qt.guiRestore(dict[0])
        for i in range(len(dict)):
            self.alg[i - 1].setGuiParams(dict[i])

    def selectiveSetGuiParams(self, dict):
        """
        Override Modules' selective Restore function

        Setup Qtickle
        selectively restore the UI, the data to do that will be in the 0th element of the dictionary
        We will then iterate through the rest of the dictionary
        Will now restore the parameters for the algorithms in the list, Each of the algs have their own selectiveSetGuiParams

        :param dict:
        :return:
        """

        self.qt = Qtickle.Qtickle(self)
        self.qt.selectiveGuiRestore(dict[0])
        for i in range(len(dict)):
            self.alg[i - 1].selectiveSetGuiParams(dict[i])

    def run(self):
        method = self.chooseAlgorithmComboBox.currentText()
        datakey = self.chooseDataComboBox.currentText()
        xvars = [str(x.text()) for x in self.xVariableList.selectedItems()]
        params, modelkey = self.getMethodParams(self.chooseAlgorithmComboBox.currentIndex())
        self.data[datakey], outlier_removal_obj = outlier_removal(self.data[datakey].df, xvars, method, params)
        self.data[datakey] = spectral_data(self.data[datakey])

    def xvar_choices(self):
        try:
            xvarchoices = self.data[self.chooseDataComboBox.currentText()].df.columns.levels[0].values
            xvarchoices = [i for i in xvarchoices if not 'Unnamed' in i]  # remove unnamed columns from choices
        except:
            xvarchoices = ['No valid choices!']
        return xvarchoices

    def hideAll(self):
        for a in self.alg:
            a.setHidden(True)

    def getMethodParams(self, index):
        return self.alg[index - 1].run()

    def outlierRemovalMethods(self):
        self.alg = []
        list_forms = [outliers_IsolationForest,
                      outliers_LOF]
        for items in list_forms:
            self.alg.append(items.Ui_Form())
            self.alg[-1].setupUi(self.Form)
            self.algorithmLayout.addWidget(self.alg[-1].get_widget())
            self.alg[-1].setHidden(True)


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = OutlierRemoval()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



from PyQt5 import QtWidgets

from point_spectra_gui.ui.StratifiedFolds import Ui_Form
from point_spectra_gui.util.Modules import Modules
from point_spectra_gui.util.spectral_data import spectral_data
from libpysat.utils.folds import stratified_folds
from libpysat.utils.utils import rows_match
import copy

class StratifiedFolds(Ui_Form, Modules):
    count = -1

    def __init__(self):
        StratifiedFolds.count += 1
        self.curr_count = StratifiedFolds.count
        print('Added StratifiedFolds with ID {}'.format(self.curr_count))

    def delete(self):
        try:
            StratifiedFolds.count -= 1
            del self.data[self.datakeys[-1]]
            del self.datakeys[-1]
        except IndexError:
            pass

    def setupUi(self, Form):
        super().setupUi(Form)
        Modules.setupUi(self, Form)

    def get_widget(self):
        return self.formGroupBox

    def connectWidgets(self):
        self.nFoldsSpinBox.setValue(2)
        self.testFoldsSpinBox.setValue(2)
        self.setComboBox(self.chooseDataToStratifyComboBox, self.datakeys)
        try:  # Some instances where perhaps there is no data to load
            data = self.data[self.chooseDataToStratifyComboBox.currentText()].df['comp'].columns.values
            self.setComboBox(self.chooseVarComboBox, data)
        except:
            pass
        self.chooseDataToStratifyComboBox.activated[int].connect(self.strat_fold_change_vars)
        self.nFoldsSpinBox.valueChanged.connect(self.strat_fold_change_testfolds)

    def setup(self):
        try:
            datakey = self.chooseDataToStratifyComboBox.currentText()

            self.data[datakey + '-Train'] = copy.deepcopy(self.data[datakey])
            self.data[datakey + '-Test'] = copy.deepcopy(self.data[datakey])
            if datakey + '-Train' not in self.datakeys and datakey + '-Test' not in self.datakeys:
                self.datakeys.append(datakey + '-Train')
                self.datakeys.append(datakey + '-Test')
        except:
            pass

    def run(self):
        datakey = self.chooseDataToStratifyComboBox.currentText()
        nfolds = self.nFoldsSpinBox.value()
        try:
            testfold = int(self.testFoldsSpinBox.value())
        except:
            testfold = 1
        colname = ('comp', self.chooseVarComboBox.currentText())
        self.data[datakey]=spectral_data(stratified_folds(self.data[datakey].df,nfolds=nfolds, sortby=colname))

        self.data[datakey + '-Train'] = spectral_data(rows_match(self.data[datakey].df,('meta', 'Folds'), [testfold], invert=True))
        self.data[datakey + '-Test'] = spectral_data(rows_match(self.data[datakey].df,('meta', 'Folds'), [testfold]))
        self.datakeys.append(datakey + '-Train')
        self.datakeys.append(datakey + '-Test')

        print(self.data.keys())
        print(self.data[datakey + '-Test'].df.index.shape)
        print(self.data[datakey + '-Train'].df.index.shape)

    def strat_fold_change_vars(self):
        self.chooseVarComboBox.clear()
        try:
            choices = self.data[self.chooseDataToStratifyComboBox.currentText()].df['comp'].columns.values
        except:
            choices = ['No composition columns!']

        self.chooseVarComboBox.addItems(choices)

    def strat_fold_change_testfolds(self):
        self.testFoldsSpinBox.setMaximum(self.nFoldsSpinBox.value())


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)

    Form = QtWidgets.QWidget()
    ui = StratifiedFolds()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



import inspect
from distutils.util import strtobool

from PyQt5 import QtCore
from PyQt5.QtWidgets import *


class Qtickle(object):
    """
    A script designed to aid in saving values inside Qt objects.
    It works by inspecting for `objectnames` and then collecting their
    values.

    The name is a portmanteau of Qt and pickle. Originally the script
    utilized pickle to save data. Now data is returned in the form of
    a dictionary. This way the user gets to choose what happens to the
    data
    """

    def __init__(self, ui):
        self.ui = ui

    def guiSave(self):
        """
        Save all values in a particular UI

        :return:
        """
        dict = {}
        # Save geometry
        # self.settings.setValue('size', self.core.size())
        # self.settings.setValue('pos', self.core.pos())
        try:
            for name, obj in inspect.getmembers(self.ui):
                if isinstance(obj, QLineEdit):
                    name = obj.objectName()
                    value = obj.text()
                    dict[name] = value

                if isinstance(obj, QCheckBox):
                    name = obj.objectName()
                    state = obj.isChecked()
                    dict[name] = state

                if isinstance(obj, QRadioButton):
                    name = obj.objectName()
                    value = obj.isChecked()  # get stored value from registry
                    dict[name] = value

                if isinstance(obj, QSpinBox):
                    name = obj.objectName()
                    value = obj.value()  # get stored value from registry
                    dict[name] = value

                if isinstance(obj, QDoubleSpinBox):
                    name = obj.objectName()
                    value = obj.value()
                    dict[name] = value

                if isinstance(obj, QSlider):
                    name = obj.objectName()
                    value = obj.value()  # get stored value from registry
                    dict[name] = value

                if isinstance(obj, QLabel):
                    name = obj.objectName()
                    value = obj.text()
                    dict[name] = value

                if isinstance(obj, QComboBox):
                    values = []  # the list that will hold all values from QCombobox
                    name = obj.objectName()  # get the QCombobox object's name
                    for i in range(obj.count()):  # QCombobox contains a number of items
                        itemData = obj.itemText(i)
                        values.append(itemData)  # put those items into a list for saving
                    index = obj.findText(obj.currentText())  # return the index of the item, assign to selected
                    dict[name + "_values"] = values  # save all the values in settings
                    dict[name + "_index"] = index  # save the indexed value in settings

                if isinstance(obj, QListWidget):
                    values = []
                    name = obj.objectName()
                    for i in range(obj.count()):
                        itemData = obj.item(i).text()
                        values.append(itemData)
                    dict[name + "_values"] = values
                    # since there is a possibility of multiple items,
                    # we'll just save the string representation of those items to be restored
                    dict[name + "_index"] = [str(x.text()) for x in obj.selectedItems()]

            return dict
        except Exception as e:
            print(e)

    def guiRestore(self, dict):
        """
        Restore the GUI. This is a hard restore, meaning that everything will be overwritten

        :param dict:
        :return:
        """
        # Restore geometry
        # self.core.resize(self.settings.value('size', QtCore.QSize(500, 500)))
        # self.core.move(self.settings.value('pos', QtCore.QPoint(60, 60)))
        for name, obj in inspect.getmembers(self.ui):
            try:
                if isinstance(obj, QLineEdit):
                    name = obj.objectName()
                    value = dict[name]
                    obj.setText(value)  # restore lineEditFile

                if isinstance(obj, QCheckBox):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        try:
                            obj.setChecked(strtobool(value))  # restore checkbox
                        except:
                            obj.setChecked(value)

                if isinstance(obj, QRadioButton):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setChecked(strtobool(value))

                if isinstance(obj, QSlider):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setValue(int(value))  # restore value from registry

                if isinstance(obj, QSpinBox):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setValue(int(value))  # restore value from registry

                if isinstance(obj, QDoubleSpinBox):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setValue(float(value))

                if isinstance(obj, QLabel):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setText(value)

                if isinstance(obj, QComboBox):
                    name = obj.objectName()
                    values = dict[name + "_values"]
                    # clear all the objects
                    # so that we don't run into issues
                    # with restoring the list of values
                    obj.clear()
                    if values is not None:
                        for value in values:
                            if not (value == '' or value == "") and len(value) > 0:
                                # if there are some values in the list, we should add them to the Combobox
                                obj.addItem(value)

                    index = dict[name + "_index"]  # next we want to select the item in question by getting it's index
                    obj.setCurrentIndex(int(index))

                if isinstance(obj, QListWidget):
                    name = obj.objectName()
                    values = dict[name + "_values"]
                    index = dict[name + "_index"]
                    obj.clear()
                    if values is not None:
                        for value in values:
                            list_item = QListWidgetItem(value)
                            obj.addItem(list_item)
                        for i in index:
                            matching_items = obj.findItems(i, QtCore.Qt.MatchExactly)
                            for item in matching_items:
                                item.setSelected(True)

            except Exception as e:
                print(e)

    def guiChanged(self, functionCall):
        """
        when the UI changes run the parameter `functionCall`

        :param functionCall:
        :return:
        """
        try:
            for name, obj in inspect.getmembers(self.ui):
                if isinstance(obj, QLineEdit):
                    obj.editingFinished.connect(lambda: functionCall())

                if isinstance(obj, QCheckBox):
                    obj.stateChanged.connect(lambda: functionCall())

                if isinstance(obj, QRadioButton):
                    obj.toggled.connect(lambda: functionCall())

                if isinstance(obj, QSpinBox):
                    obj.valueChanged.connect(lambda: functionCall())

                if isinstance(obj, QDoubleSpinBox):
                    obj.valueChanged.connect(lambda: functionCall())

                if isinstance(obj, QSlider):
                    obj.event.connect(lambda: functionCall())

                if isinstance(obj, QComboBox):
                    obj.currentIndexChanged.connect(lambda: functionCall())

                    # if isinstance(obj, QListWidget): This needs to be added at somepoint
                    #     obj.

        except Exception as e:
            print(e)

    def guiDisonnect(self):
        """
        This is mainly a problem with the buttons that were opening file dialogs, but if there are other cases,
        please consider adding those cases to this method.

        https://stackoverflow.com/questions/3530590/qt-signals-and-slot-connected-twice-what-happens#_=_
        Just like in this stackoverflow question, there are some instances where the UI will call connects
        more than once... this is dangerous as it causes dialog boxes to display also more than once.
        This method is an attempt to cull this problem.

        :return:
        """
        try:
            for name, obj in inspect.getmembers(self.ui):
                if isinstance(obj, QPushButton):
                    obj.disconnect()

        except Exception as e:
            print(e)

    def selectiveGuiRestore(self, dict):
        """
        Restore the GUI. This is a softer restore in regards to comboboxes.
        Comboboxes will not be overwritten but instead will be given an index to set themselves to.
        Everything else will continue to be overwritten, be careful.


        :param dict:
        :return:
        """
        for name, obj in inspect.getmembers(self.ui):
            pass
            try:
                if isinstance(obj, QCheckBox):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        try:
                            obj.setChecked(strtobool(value))  # restore checkbox
                        except:
                            obj.setChecked(value)

                if isinstance(obj, QRadioButton):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setChecked(strtobool(value))

                if isinstance(obj, QSlider):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setValue(int(value))  # restore value from registry

                if isinstance(obj, QSpinBox):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setValue(int(value))  # restore value from registry

                if isinstance(obj, QDoubleSpinBox):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setValue(float(value))

                if isinstance(obj, QComboBox):
                    name = obj.objectName()
                    # select the item in question by setting it's index
                    index = dict[name + "_index"]
                    obj.setCurrentIndex(int(index))

                if isinstance(obj, QListWidget):
                    name = obj.objectName()
                    index = dict[name + "_index"]
                    for i in index:
                        matching_items = obj.findItems(i, QtCore.Qt.MatchExactly)
                        for item in matching_items:
                            item.setSelected(True)

            except Exception as e:
                print(e)



from PyQt5 import QtWidgets

from point_spectra_gui.ui.MultiplyByVector import Ui_Form
from point_spectra_gui.util.Modules import Modules
from point_spectra_gui.util.spectral_data import spectral_data
from libpysat.transform.multiply_vector import multiply_vector

class MultiplyByVector(Ui_Form, Modules):
    def setupUi(self, Form):
        super().setupUi(Form)
        Modules.setupUi(self, Form)

    def get_widget(self):
        return self.groupBox

    def connectWidgets(self):
        self.setComboBox(self.chooseDataComboBox, self.datakeys)
        self.pushButton.clicked.connect(lambda: self.on_getDataButton_clicked(self.vectorFileLineEdit))

    def run(self):
        datakey = self.chooseDataComboBox.currentText()
        vectorfile = self.vectorFileLineEdit.text()

        try:
            self.data[datakey] = spectral_data(multiply_vector(self.data[datakey],vectorfile))
        except Exception as e:
            print(e)

    def on_getDataButton_clicked(self, lineEdit):
        filename, _filter = QtWidgets.QFileDialog.getOpenFileName(None, "Open Vector Data File", '.', "(*.csv)")
        lineEdit.setText(filename)
        if lineEdit.text() == "":
            lineEdit.setText("*.csv")


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)

    Form = QtWidgets.QWidget()
    ui = MultiplyByVector()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Automatically generated - don't edit.
# Use `python setup.py build_ui` to update it.

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tableWidget = QtWidgets.QTableWidget(Form)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setColumnCount(2)
        self.tableWidget.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(1, item)
        self.verticalLayout.addWidget(self.tableWidget)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(("Shortcut"))
        item = self.tableWidget.horizontalHeaderItem(0)
        item.setText(("Descrption"))
        item = self.tableWidget.horizontalHeaderItem(1)
        item.setText(("New Column"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())




# -*- coding: utf-8 -*-

# Automatically generated - don't edit.
# Use `python setup.py build_ui` to update it.

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formGroupBox = QtWidgets.QGroupBox(Form)
        self.formGroupBox.setObjectName("formGroupBox")
        self.formLayout = QtWidgets.QFormLayout(self.formGroupBox)
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.chooseModelLabel = QtWidgets.QLabel(self.formGroupBox)
        self.chooseModelLabel.setObjectName("chooseModelLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.chooseModelLabel)
        self.chooseModelComboBox = QtWidgets.QComboBox(self.formGroupBox)
        self.chooseModelComboBox.setObjectName("chooseModelComboBox")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.chooseModelComboBox)
        self.chooseDataLabel = QtWidgets.QLabel(self.formGroupBox)
        self.chooseDataLabel.setObjectName("chooseDataLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.chooseDataLabel)
        self.chooseDataListWidget = QtWidgets.QListWidget(self.formGroupBox)
        self.chooseDataListWidget.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.chooseDataListWidget.setObjectName("chooseDataListWidget")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.chooseDataListWidget)
        self.verticalLayout.addWidget(self.formGroupBox)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(("Form"))
        self.formGroupBox.setTitle(("Regression - Predict"))
        self.chooseModelLabel.setText(("Choose model:"))
        self.chooseDataLabel.setText(("Choose data:"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())




import numpy as np
import pandas as pd
from PyQt5 import QtWidgets
from point_spectra_gui.util import Qtickle
from libpysat.regression import regression
from point_spectra_gui.util.spectral_data import spectral_data
from point_spectra_gui.core.regressionMethods import *
from point_spectra_gui.ui.RegressionTrain import Ui_Form
from point_spectra_gui.util.Modules import Modules


class RegressionTrain(Ui_Form, Modules):
    count = -1

    def __init__(self):
        RegressionTrain.count += 1
        self.curr_count = RegressionTrain.count

    def delete(self):
        try:
            RegressionTrain.count -= 1
            del self.models[self.modelkeys[-1]]
            del self.modelkeys[-1]
        except:
            pass

        RegressionTrain.count -= 1
        self.modelkeys = self.modelkeys[:-1]

    def setupUi(self, Form):
        self.Form = Form
        super().setupUi(Form)
        Modules.setupUi(self, Form)
        self.regressionMethods()

    def get_widget(self):
        return self.groupLayout

    def make_regression_widget(self, alg, params=None):
        self.hideAll()
        print(alg)
        try:
            self.alg[alg].setHidden(False)
        except:
            pass

    def connectWidgets(self):
        self.algorithm_list = ['Choose an algorithm',
                               'PLS',
                               'OLS',
                               'OMP',
                               'LASSO',
                               'Elastic Net',
                               'Ridge',
                               'BRR',
                               'ARD',
                               'LARS',
                               # 'LASSO LARS', - This is having issues. Hide until we can debug
                               'SVR']
        self.setComboBox(self.chooseDataComboBox, self.datakeys)
        self.setComboBox(self.chooseAlgorithmComboBox, self.algorithm_list)
        self.yMaxDoubleSpinBox.setMaximum(999999)
        self.yMinDoubleSpinBox.setMaximum(999999)
        self.yMaxDoubleSpinBox.setValue(100)
        self.changeComboListVars(self.yVariableList, self.yvar_choices())
        self.changeComboListVars(self.xVariableList, self.xvar_choices())
        self.xvar_choices()
        self.chooseAlgorithmComboBox.currentIndexChanged.connect(
            lambda: self.make_regression_widget(self.chooseAlgorithmComboBox.currentText()))
        self.chooseDataComboBox.currentIndexChanged.connect(
            lambda: self.changeComboListVars(self.yVariableList, self.yvar_choices()))
        self.chooseDataComboBox.currentIndexChanged.connect(
            lambda: self.changeComboListVars(self.xVariableList, self.xvar_choices()))

    def getGuiParams(self):
        """
        Overriding Modules' getGuiParams, because I'll need to do a list of lists
        in order to obtain the regressionMethods' parameters
        """
        self.qt = Qtickle.Qtickle(self)
        s = []
        s.append(self.qt.guiSave())
        for items in self.alg:
            s.append(self.alg[items].getGuiParams())
        return s

    def setGuiParams(self, dict):
        """
        Overriding Modules' setGuiParams as we are using a list of lists to

        :param dict:
        :return:
        """

        self.qt = Qtickle.Qtickle(self)
        self.qt.guiRestore(dict[0])
        keys = list(self.alg.keys())
        for i in range(len(dict)):
            self.alg[keys[i - 1]].setGuiParams(dict[i])

    def selectiveSetGuiParams(self, dict):
        """
        Override Modules' selective Restore function

        Setup Qtickle
        selectively restore the UI, the data to do that will be in the 0th element of the dictionary
        We will then iterate through the rest of the dictionary
        Will now restore the parameters for the algorithms in the list, Each of the algs have their own selectiveSetGuiParams

        :param dict:
        :return:
        """

        self.qt = Qtickle.Qtickle(self)
        self.qt.selectiveGuiRestore(dict[0])
        keys = list(self.alg.keys())
        for i in range(len(dict)):
            self.alg[keys[i - 1]].selectiveSetGuiParams(dict[i])

    def setup(self):
        method = self.chooseAlgorithmComboBox.currentText()
        xvars = [str(x.text()) for x in self.xVariableList.selectedItems()]
        yvars = [('comp', str(y.text())) for y in self.yVariableList.selectedItems()]
        yrange = [self.yMinDoubleSpinBox.value(), self.yMaxDoubleSpinBox.value()]
        try:
            params, modelkey = self.alg[self.chooseAlgorithmComboBox.currentText()].run()
            modelkey = "{} - {} - ({}, {}) {}".format(method, yvars[0][-1], yrange[0], yrange[1], modelkey)
            self.list_amend(self.modelkeys, self.curr_count, modelkey)
            print(params, modelkey)
            self.models[modelkey] = regression.regression([method], [yrange], [params])
            self.model_xvars[modelkey] = xvars
            self.model_yvars[modelkey] = yvars
        except:
            pass

    def run(self):
        method = self.chooseAlgorithmComboBox.currentText()
        datakey = self.chooseDataComboBox.currentText()
        xvars = [str(x.text()) for x in self.xVariableList.selectedItems()]
        yvars = [('comp', str(y.text())) for y in self.yVariableList.selectedItems()]
        yrange = [self.yMinDoubleSpinBox.value(), self.yMaxDoubleSpinBox.value()]

        params, modelkey = self.alg[self.chooseAlgorithmComboBox.currentText()].run()
        modelkey = "{} - {} - ({}, {}) {}".format(method, yvars[0][-1], yrange[0], yrange[1], modelkey)
        self.list_amend(self.modelkeys, self.curr_count, modelkey)
        print(params, modelkey)
        self.models[modelkey] = regression.regression([method], [yrange], [params])
        x = self.data[datakey].df[xvars]
        y = self.data[datakey].df[yvars]
        x = np.array(x)
        y = np.array(y)
        ymask = np.squeeze((y > yrange[0]) & (y < yrange[1]))
        y = y[ymask]
        x = x[ymask, :]
        self.models[modelkey].fit(x, y)
        self.model_xvars[modelkey] = xvars
        self.model_yvars[modelkey] = yvars
        try:
            coef = np.squeeze(self.models[modelkey].model.coef_)
            coef = pd.DataFrame(coef)
            coef.index = pd.MultiIndex.from_tuples(self.data[datakey].df[xvars].columns.values)
            coef = coef.T
            coef[('meta', 'Model')] = modelkey
            try:
                coef[('meta', 'Intercept')] = self.models[modelkey].model.intercept_
            except:
                pass
            try:
                self.data['Model Coefficients'] = spectral_data(pd.concat([self.data['Model Coefficients'].df, coef]))
            except:
                self.data['Model Coefficients'] = spectral_data(coef)
                self.datakeys.append('Model Coefficients')
        except:
            pass

    def yvar_choices(self):
        try:
            yvarchoices = self.data[self.chooseDataComboBox.currentText()].df['comp'].columns.values
            yvarchoices = [i for i in yvarchoices if not 'Unnamed' in i]  # remove unnamed columns from choices
        except:
            yvarchoices = ['No composition columns!']
        return yvarchoices

    def xvar_choices(self):
        try:
            xvarchoices = self.data[self.chooseDataComboBox.currentText()].df.columns.levels[0].values
            xvarchoices = [i for i in xvarchoices if not 'Unnamed' in i]  # remove unnamed columns from choices
        except:
            xvarchoices = ['No valid choices!']
        return xvarchoices

    def hideAll(self):
        for a in self.alg:
            self.alg[a].setHidden(True)

    def regressionMethods(self):
        self.alg = {'ARD': ARD.Ui_Form(),
                    'BRR': BayesianRidge.Ui_Form(),
                    'Elastic Net': ElasticNet.Ui_Form(),
                    'GP': GP.Ui_Form(),
                    # 'KRR': KRR.Ui_Form(),
                    'LARS': LARS.Ui_Form(),
                    'LASSO': Lasso.Ui_Form(),
                    # 'LASSO LARS': LassoLARS.Ui_Form(),
                    'OLS': OLS.Ui_Form(),
                    'OMP': OMP.Ui_Form(),
                    'PLS': PLS.Ui_Form(),
                    'Ridge': Ridge.Ui_Form(),
                    'SVR': SVR.Ui_Form()
                    }

        for item in self.alg:
            self.alg[item].setupUi(self.Form)
            self.algorithmLayout.addWidget(self.alg[item].get_widget())
            self.alg[item].setHidden(True)


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = RegressionTrain()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



from PyQt5 import QtWidgets

from point_spectra_gui.ui.DataTable import Ui_Form
from point_spectra_gui.util.PandasModel import PandasModel
from point_spectra_gui.util.Modules import Modules
from point_spectra_gui.util.Worker import Worker


class DataTable(QtWidgets.QWidget, Ui_Form, Modules):
    """
    Displays the data stored inside the memory of this application
    """

    def __init__(self, *args, **kwargs):
        QtWidgets.QWidget.__init__(self, *args, **kwargs)
        self.setupUi(self)
        self.connectWidgets()
        self.refreshTable = Worker(self.on_refreshTable)

    def get_widget(self):
        return self.dockWidget

    def connectWidgets(self):
        self.setComboBox(self.chooseDataComboBox, self.datakeys)
        self.chooseDataComboBox.currentIndexChanged.connect(lambda: self.on_refreshTable())
        self.refreshDataPushButton.clicked.connect(lambda: self.connectWidgets())
        self.refreshTablePushButton.clicked.connect(lambda: self.on_refreshTable())

    def on_refreshTable(self):
        try:
            pandasModel = PandasModel(self.data[self.chooseDataComboBox.currentText()].df)
            self.tableView.setModel(pandasModel)
        except:
            print('KeyError: \'\': Cannot read from an empty string')


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = DataTable()
    Form.show()
    sys.exit(app.exec_())




# -*- coding: utf-8 -*-

# Automatically generated - don't edit.
# Use `python setup.py build_ui` to update it.

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox = QtWidgets.QGroupBox(Form)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.Ridge = QtWidgets.QGroupBox(self.groupBox)
        self.Ridge.setEnabled(True)
        self.Ridge.setObjectName("Ridge")
        self.formLayout_2 = QtWidgets.QFormLayout(self.Ridge)
        self.formLayout_2.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout_2.setObjectName("formLayout_2")
        self.alphaLabel = QtWidgets.QLabel(self.Ridge)
        self.alphaLabel.setObjectName("alphaLabel")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.alphaLabel)
        self.alphaDoubleSpinBox = QtWidgets.QDoubleSpinBox(self.Ridge)
        self.alphaDoubleSpinBox.setProperty("value", 1.0)
        self.alphaDoubleSpinBox.setObjectName("alphaDoubleSpinBox")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.alphaDoubleSpinBox)
        self.fitInterceptLabel = QtWidgets.QLabel(self.Ridge)
        self.fitInterceptLabel.setObjectName("fitInterceptLabel")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.fitInterceptLabel)
        self.fitInterceptCheckBox = QtWidgets.QCheckBox(self.Ridge)
        self.fitInterceptCheckBox.setChecked(True)
        self.fitInterceptCheckBox.setObjectName("fitInterceptCheckBox")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.fitInterceptCheckBox)
        self.maxNumOfIterationsLabel = QtWidgets.QLabel(self.Ridge)
        self.maxNumOfIterationsLabel.setObjectName("maxNumOfIterationsLabel")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.maxNumOfIterationsLabel)
        self.maxNumOfIterationslineEdit = QtWidgets.QLineEdit(self.Ridge)
        self.maxNumOfIterationslineEdit.setObjectName("maxNumOfIterationslineEdit")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.maxNumOfIterationslineEdit)
        self.normalizeLabel = QtWidgets.QLabel(self.Ridge)
        self.normalizeLabel.setObjectName("normalizeLabel")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.normalizeLabel)
        self.normalizeCheckBox = QtWidgets.QCheckBox(self.Ridge)
        self.normalizeCheckBox.setObjectName("normalizeCheckBox")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.normalizeCheckBox)
        self.toleranceLabel = QtWidgets.QLabel(self.Ridge)
        self.toleranceLabel.setObjectName("toleranceLabel")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.toleranceLabel)
        self.toleranceDoubleSpinBox = QtWidgets.QDoubleSpinBox(self.Ridge)
        self.toleranceDoubleSpinBox.setObjectName("toleranceDoubleSpinBox")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.toleranceDoubleSpinBox)
        self.horizontalLayout.addWidget(self.Ridge)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.formGroupBox_2 = QtWidgets.QGroupBox(self.groupBox)
        self.formGroupBox_2.setObjectName("formGroupBox_2")
        self.formLayout_5 = QtWidgets.QFormLayout(self.formGroupBox_2)
        self.formLayout_5.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout_5.setObjectName("formLayout_5")
        self.verticalLayout.addWidget(self.formGroupBox_2)
        self.verticalLayout_2.addWidget(self.groupBox)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(("Form"))
        self.Ridge.setTitle(("Ridge"))
        self.alphaLabel.setText(("Alpha"))
        self.alphaDoubleSpinBox.setToolTip(_translate("Form", "Regularization strength; must be a positive float. Regularization\n"
"improves the conditioning of the problem and reduces the variance of\n"
"the estimates. Larger values specify stronger regularization. Alpha\n"
"corresponds to C^-1 in other linear models such as LogisticRegression\n"
"or LinearSVC. If an array is passed, penalties are assumed to be specific\n"
"to the targets. Hence they must correspond in number.\n"
""))
        self.alphaDoubleSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.Ridge.html"))
        self.fitInterceptLabel.setText(("Fit Intercept"))
        self.fitInterceptCheckBox.setToolTip(_translate("Form", "Whether to calculate the intercept for this model. If set to false,\n"
"no intercept will be used in calculations (e.g. data is expected to\n"
"be already centered)."))
        self.fitInterceptCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.Ridge.html"))
        self.maxNumOfIterationsLabel.setText(("Max # of Iterations"))
        self.maxNumOfIterationslineEdit.setToolTip(_translate("Form", "Maximum number of iterations for conjugate gradient solver. For\n"
"\'sparse_cg\' and \'lsqr\' solvers, the default value is determined by\n"
"scipy.sparse.linalg. For \'sag\' solver, the default value is 1000.\n"
""))
        self.maxNumOfIterationslineEdit.setText(("None"))
        self.normalizeLabel.setText(("Normalize"))
        self.normalizeCheckBox.setToolTip(_translate("Form", "This parameter is ignored when fit_intercept is set to False. If True,\n"
"the regressors X will be normalized before regression by subtracting\n"
"the mean and dividing by the l2-norm. If you wish to standardize,\n"
"please use sklearn.preprocessing.StandardScaler before calling fit on\n"
"an estimator with normalize=False."))
        self.normalizeCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.Ridge.html"))
        self.toleranceLabel.setText(("Tolerance"))
        self.toleranceDoubleSpinBox.setToolTip(("Precision of the solution."))
        self.toleranceDoubleSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.Ridge.html"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())




from PyQt5 import QtWidgets

from point_spectra_gui.__init__ import __version__
from point_spectra_gui.ui.About import Ui_Form


class About(QtWidgets.QWidget, Ui_Form):
    """
    Display the version for the User. Good for debugging purposes
    """

    def __init__(self, *args, **kwargs):
        QtWidgets.QWidget.__init__(self, *args, **kwargs)
        self.setupUi(self)
        self.versionLabel.setText("Version " + __version__)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    form = About()
    form.show()
    sys.exit(app.exec_())




# -*- coding: utf-8 -*-
"""
Created on Thu Sep  1 13:09:21 2016

@author: rbanderson
"""
import numpy as np
from matplotlib import pyplot as plot

from point_spectra_gui.util import colormaps


def cmaps():
    plot.register_cmap(name='viridis', cmap=colormaps.viridis)
    plot.register_cmap(name='magma', cmap=colormaps.magma)
    plot.register_cmap(name='inferno', cmap=colormaps.inferno)
    plot.register_cmap(name='plasma', cmap=colormaps.plasma)


def make_plot(x, y, figpath, figfile=None, xrange=None, yrange=None, xtitle='Reference (wt.%)',
              ytitle='Prediction (wt.%)', title=None,
              lbl='', one_to_one=False, rmse=True, dpi=1000, color=None, annot_mask=None, cmap=None, colortitle='',
              loadfig=None, masklabel='', marker='o', linestyle='None', hline=None, hlinelabel=None, hlinestyle='--',
              yzero=False, linewidth=1.0, vlines=None):
    if loadfig is not None:
        fig = loadfig
        axes = fig.gca()
    else:
        fig = plot.figure()

        axes = fig.gca()
        if title:
            fig.suptitle(title)
        if xtitle:
            axes.set_xlabel(xtitle)
        if ytitle:
            axes.set_ylabel(ytitle)
        if xrange:
            axes.set_xlim(xrange)
            xind = np.where((x > xrange[0]) & (x < xrange[1]))
        if yrange:
            axes.set_ylim(yrange)
        else:
            yrange = [np.min(np.array(y)[xind]), np.max(np.array(y)[xind])]
            axes.set_ylim(yrange)
    if hline:
        axes.axhline(hline, color='k', label=hlinelabel, linestyle=hlinestyle)
    if vlines:
        for x in vlines:
            axes.axvline(x, color='k', linestyle='--')
    if one_to_one:
        axes.plot([0, 100], [0, 100], color='k')
        if rmse:
            rmse_val = np.sqrt(np.mean((y - x) ** 2))
            lbl = lbl + ' (RMSE=' + str(round(rmse_val, 2)) + ')'

    if cmap is not None:
        axes.plot(x, y, c=color, cmap=cmap, marker=marker, markeredgecolor='Black', markeredgewidth=0.25)
        axes.colorbar(label=colortitle)
    else:
        axes.plot(x, y, color=color, label=lbl, marker=marker, ls=linestyle, linewidth=linewidth,
                  markeredgecolor='Black', markeredgewidth=0.25)

        if annot_mask is not None:
            axes.plot(x[annot_mask], y[annot_mask], facecolors='none', linewidth=linewidth, label=masklabel,
                      marker=marker, markeredgecolor='Black', markeredgewidth=2)

    if yzero:
        axes.set_ylim(bottom=0)

    axes.legend(loc='best', fontsize=8, scatterpoints=1, numpoints=1)
    if figpath and figfile:
        fig.savefig(figpath + '/' + figfile, dpi=dpi)
    return fig


def pca_ica_plot(data, x_component, y_component, colorvar=None, cmap='viridis', method='PCA', figpath=None,
                 figfile=None):
    cmaps()
    x_label = ''
    y_label = ''
    x = [data.df[(method, x_component)]]
    y = [data.df[(method, y_component)]]
    if method == 'PCA':
        x_loading = data.dim_red.components_[int(x_component) - 1, :]
        y_loading = data.dim_red.components_[int(y_component) - 1, :]

        x_variance = data.dim_red.explained_variance_ratio_[int(x_component) - 1] * 100
        y_variance = data.dim_red.explained_variance_ratio_[int(y_component) - 1] * 100
        x_label = 'PC ' + str(x_component) + ' (' + str(round(x_variance, 1)) + r'%)'
        y_label = 'PC ' + str(y_component) + ' (' + str(round(y_variance, 1)) + r'%)'
    if method == 'FastICA':
        x_loading = data.dim_red.components_[int(x_component) - 1, :]
        y_loading = data.dim_red.components_[int(y_component) - 1, :]
        x_label = 'Source ' + str(x_component)
        y_label = 'Source ' + str(y_component)

    if method == 'JADE-ICA':
        x_loading = data.dim_red.ica_jade_loadings[int(x_component) - 1, :].T
        y_loading = data.dim_red.ica_jade_loadings[int(y_component) - 1, :].T
        x_label = 'Source ' + str(x_component)
        y_label = 'Source ' + str(y_component)

    # set up the subplots
    fig = plot.figure()
    fig.set_size_inches(10, 4)
    ax1 = fig.add_subplot(2, 2, (1, 3))
    ax2 = fig.add_subplot(2, 2, 2)
    ax3 = fig.add_subplot(2, 2, 4, xlabel='Wavelength (nm)')

    ax1.set_xlabel(x_label)
    ax1.set_ylabel(y_label)

    if colorvar:
        try:
            mappable = ax1.scatter(np.squeeze(x), np.squeeze(y), c=data.df[('comp', colorvar)], cmap=cmap, linewidth=0.2, edgecolor='Black')
        except:
            mappable = ax1.scatter(np.squeeze(x), np.squeeze(y), c=data.df[('meta', colorvar)], cmap=cmap, linewidth=0.2, edgecolor='Black')
            # TODO: handle any top-level label for colorval, not just comp or meta
        fig.colorbar(mappable, label=colorvar, ax=ax1)
    else:
        ax1.scatter(x, y, linewidth=0.2, edgecolor='Black')

    # plot the loadings
    wvls = data.df['wvl'].columns.values
    ax2.plot(wvls, x_loading, linewidth=0.5)
    ax3.plot(wvls, y_loading, linewidth=0.5)

    ax2.set_yticklabels([])
    ax2.set_xticklabels([])
    ax2.set_ylabel(x_label)

    ax3.set_yticklabels([])
    ax3.set_ylabel(y_label)

    fig.subplots_adjust(hspace=0)

    #save the loadings to a file
    outloadings = np.array([wvls, x_loading, y_loading])


    if figpath and figfile:
        fig.savefig(figpath + '\\' + figfile, dpi=1000)



from PyQt5 import QtWidgets, QtCore
from sklearn.linear_model.ridge import Ridge
from sklearn.linear_model.ridge import RidgeCV

from point_spectra_gui.ui.cv_Ridge import Ui_Form
from point_spectra_gui.util.Modules import Modules
import numpy as np

class Ui_Form(Ui_Form, Ridge, RidgeCV, Modules):
    def setupUi(self, Form):
        super().setupUi(Form)
        self.checkMinAndMax()
        self.connectWidgets()

    def get_widget(self):
        return self.Ridge

    def setHidden(self, bool):
        self.get_widget().setHidden(bool)

    def connectWidgets(self):
        ridge = Ridge()
        self.min_alpha_doubleSpinBox.setDecimals(20)
        self.max_alpha_doubleSpinBox.setDecimals(20)

        self.min_alpha_doubleSpinBox.setValue(0.000000000000001)
        self.max_alpha_doubleSpinBox.setValue(0.01)
        self.n_alpha_spinBox.setValue(100)
        self.fit_intercept_list.setCurrentItem(
            self.fit_intercept_list.findItems(str(ridge.fit_intercept), QtCore.Qt.MatchExactly)[0])
        self.normalize_list.setCurrentItem(
            self.normalize_list.findItems(str(ridge.normalize), QtCore.Qt.MatchExactly)[0])

    def run(self):
        fit_intercept_items = [i.text() == 'True' for i in self.fit_intercept_list.selectedItems()]
        normalize_items = [i.text() == 'True' for i in self.normalize_list.selectedItems()]

        alphas = np.logspace(np.log10(self.min_alpha_doubleSpinBox.value()), np.log10(self.max_alpha_doubleSpinBox.value()),
                             num=self.n_alpha_spinBox.value())
        params = {'alpha': alphas,
                  'fit_intercept': fit_intercept_items,
                  'normalize': normalize_items}
        modelkey = str(params)
        return params, modelkey


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



import pandas as pd
from PyQt5 import QtWidgets
from point_spectra_gui.util.io import io_ccam_pds
from point_spectra_gui.util.spectral_data import spectral_data

from point_spectra_gui.ui.ReadChemCamData import Ui_Form
from point_spectra_gui.util.Modules import Modules


class ReadChemCamData(Ui_Form, Modules):
    def setupUi(self, Form):
        super().setupUi(Form)
        Modules.setupUi(self, Form)

    def get_widget(self):
        return self.groupBox

    def connectWidgets(self):
        self.searchStringLineEdit.setText("*ccs*.csv")
        self.lookup_col.setText('Spacecraft Clock')
        self.data_col.setText('sclock')
        self.searchDirectorypushButton.clicked.connect(self.on_searchpathButton_clicked)
        self.metadatapushButton.clicked.connect(self.on_metadataButton_clicked)
        self.hide_col_options()
        self.col_check.stateChanged.connect(self.hide_col_options)
    def hide_col_options(self):
        if self.col_check.isChecked():
            self.column_explain.setHidden(False)
            self.data_col.setHidden(False)
            self.lookup_col.setHidden(False)
            self.data_col_label.setHidden(False)
            self.lookup_col_label.setHidden(False)
            self.line.setHidden(False)
        else:
            self.column_explain.setHidden(True)
            self.data_col.setHidden(True)
            self.lookup_col.setHidden(True)
            self.data_col_label.setHidden(True)
            self.lookup_col_label.setHidden(True)
            self.line.setHidden(True)

    def on_searchpathButton_clicked(self):
        dirname = QtWidgets.QFileDialog.getExistingDirectory(parent=None, caption="Select Search Directory",
                                                             directory='.')
        self.searchDirectoryLineEdit.setText(dirname)
        if self.searchDirectoryLineEdit.text() == "":
            self.searchDirectoryLineEdit.setText("*/")

    def on_metadataButton_clicked(self):
        self.metadata_file, null = QtWidgets.QFileDialog.getOpenFileNames(parent=None,
                                                                          caption="Select metadata file(s)",
                                                                          directory='.')
        self.metadataFilesLineEdit.setText(str(self.metadata_file))
        if self.metadataFilesLineEdit.text() == "":
            self.metadataFilesLineEdit.setText("*/")

    def setup(self):
        # TODO this file needs to be redone to fit the similar setup to `LoadData`
        pass


    def run(self):
        params = self.getGuiParams()
        searchdir = self.searchDirectoryLineEdit.text()
        searchstring = self.searchStringLineEdit.text()
        to_csv = self.outputFileNameLineEdit.text()
        left_on = self.data_col.text()
        right_on = self.lookup_col.text()

        try:
            lookupfile = self.metadataFilesLineEdit.text()
            lookupfile = lookupfile[2:-2].split(',')
            lookupfile = [f.strip('\\\' "') for f in lookupfile]
            if lookupfile[0] == '':
                lookupfile = None
        except:
            lookupfile = None
        ave = self.averagesradioButton.isChecked()
        progressbar = QtWidgets.QProgressDialog()
        io_ccam_pds.ccam_batch(searchdir, searchstring=searchstring, to_csv=Modules.outpath + '/' + to_csv,
                               lookupfile=lookupfile, ave=ave, progressbar=progressbar, left_on=left_on, right_on=right_on)
        self.do_get_data(Modules.outpath + '/' + to_csv, 'ChemCam')

    def do_get_data(self, filename, keyname):
        try:
            print('Loading data file: ' + str(filename))
            self.data[keyname] = spectral_data(pd.read_csv(filename, header=[0, 1]))
            self.datakeys.append(keyname)
        except Exception as e:
            print('Problem reading data: {}'.format(e))


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    
    Form = QtWidgets.QWidget()
    ui = ReadChemCamData()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



from PyQt5 import QtCore

from PyQt5.QtCore import QThread


class Worker(QThread):
    taskFinished = QtCore.pyqtSignal()

    def __init__(self, func, parent=None):
        QThread.__init__(self, parent)
        self.func = func

    def __del__(self):
        self.wait()

    def run(self):
        self.func()



import numpy as np
from PyQt5 import QtWidgets, QtCore
from sklearn.linear_model import ElasticNet

from point_spectra_gui.ui.cv_ElasticNet import Ui_Form
from point_spectra_gui.util.Modules import Modules


class Ui_Form(Ui_Form, ElasticNet, Modules):
    def setupUi(self, Form):
        super().setupUi(Form)
        self.checkMinAndMax()
        self.connectWidgets()

    def get_widget(self):
        return self.elasticNetGroupBox

    def setHidden(self, bool):
        self.get_widget().setHidden(bool)

    def connectWidgets(self):
        en = ElasticNet()
        self.minalpha_spin.setDecimals(20)
        self.maxalpha_spin.setDecimals(20)

        self.minalpha_spin.setValue(0.000000000000001)
        self.maxalpha_spin.setValue(0.01)
        self.nalpha_spin.setValue(100)

        self.enl1_ratioLineEdit.setText('0.1, 0.5, 0.7, 0.9, 0.95, 0.99, 1.0')
        self.enfit_intercept_list.setCurrentItem(
            self.enfit_intercept_list.findItems(str(en.fit_intercept), QtCore.Qt.MatchExactly)[0])
        self.enmax_iterLineEdit.setText(str(en.max_iter))
        self.entolLineEdit.setText(str(en.tol))
        self.enpositive_list.setCurrentItem(self.enpositive_list.findItems(str(en.positive), QtCore.Qt.MatchExactly)[0])

    def run(self):
        fit_intercept_items = [i.text() == 'True' for i in self.enfit_intercept_list.selectedItems()]
        positive_items = [i.text() == 'True' for i in self.enpositive_list.selectedItems()]
        alphas = np.logspace(np.log10(self.minalpha_spin.value()), np.log10(self.maxalpha_spin.value()),
                             num=self.nalpha_spin.value())
        params = {
            'alpha': alphas,
            'l1_ratio': [float(i) for i in self.enl1_ratioLineEdit.text().split(',')],
            'fit_intercept': fit_intercept_items,
            'max_iter': [int(i) for i in self.enmax_iterLineEdit.text().split(',')],
            'tol': [float(i) for i in self.entolLineEdit.text().split(',')],
            'precompute': [True],
            'copy_X': [True],
            'positive': positive_items,
            'selection': ['random']}

        modelkey = str(params)
        return params, modelkey


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



from PyQt5 import QtWidgets
from point_spectra_gui.ui.AboutQt import Ui_AboutQT


class AboutQT(QtWidgets.QWidget, Ui_AboutQT):
    """
    Display the version for the User. Good for debugging purposes
    """

    def __init__(self, *args, **kwargs):
        QtWidgets.QWidget.__init__(self, *args, **kwargs)
        self.setupUi(self)


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    form = AboutQT()
    form.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Automatically generated - don't edit.
# Use `python setup.py build_ui` to update it.

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox = QtWidgets.QGroupBox(Form)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.maxLabel_7 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_7.setObjectName("maxLabel_7")
        self.gridLayout_2.addWidget(self.maxLabel_7, 10, 4, 1, 1)
        self.minSpinBox = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox.setObjectName("minSpinBox")
        self.gridLayout_2.addWidget(self.minSpinBox, 4, 3, 1, 1)
        self.PredictionComboBox = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox.setObjectName("PredictionComboBox")
        self.gridLayout_2.addWidget(self.PredictionComboBox, 4, 1, 1, 1)
        self.PredictionComboBox_15 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_15.setObjectName("PredictionComboBox_15")
        self.gridLayout_2.addWidget(self.PredictionComboBox_15, 18, 1, 1, 1)
        self.maxSpinBox_5 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_5.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_5.setObjectName("maxSpinBox_5")
        self.gridLayout_2.addWidget(self.maxSpinBox_5, 8, 5, 1, 1)
        self.referencePredictionComboBox = QtWidgets.QComboBox(self.groupBox)
        self.referencePredictionComboBox.setObjectName("referencePredictionComboBox")
        self.gridLayout_2.addWidget(self.referencePredictionComboBox, 1, 1, 1, 1)
        self.maxSpinBox_12 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_12.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_12.setObjectName("maxSpinBox_12")
        self.gridLayout_2.addWidget(self.maxSpinBox_12, 15, 5, 1, 1)
        self.PredictionComboBox_12 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_12.setObjectName("PredictionComboBox_12")
        self.gridLayout_2.addWidget(self.PredictionComboBox_12, 15, 1, 1, 1)
        self.PredictionComboBox_8 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_8.setObjectName("PredictionComboBox_8")
        self.gridLayout_2.addWidget(self.PredictionComboBox_8, 11, 1, 1, 1)
        self.maxSpinBox_15 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_15.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_15.setObjectName("maxSpinBox_15")
        self.gridLayout_2.addWidget(self.maxSpinBox_15, 18, 5, 1, 1)
        self.maxLabel_14 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_14.setObjectName("maxLabel_14")
        self.gridLayout_2.addWidget(self.maxLabel_14, 17, 4, 1, 1)
        self.maxLabel_2 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_2.setObjectName("maxLabel_2")
        self.gridLayout_2.addWidget(self.maxLabel_2, 5, 4, 1, 1)
        self.PredictionComboBox_2 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_2.setObjectName("PredictionComboBox_2")
        self.gridLayout_2.addWidget(self.PredictionComboBox_2, 5, 1, 1, 1)
        self.highPredictionComboBox = QtWidgets.QComboBox(self.groupBox)
        self.highPredictionComboBox.setObjectName("highPredictionComboBox")
        self.gridLayout_2.addWidget(self.highPredictionComboBox, 19, 1, 1, 1)
        self.minLabel_6 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_6.setObjectName("minLabel_6")
        self.gridLayout_2.addWidget(self.minLabel_6, 9, 2, 1, 1)
        self.minLabel = QtWidgets.QLabel(self.groupBox)
        self.minLabel.setObjectName("minLabel")
        self.gridLayout_2.addWidget(self.minLabel, 4, 2, 1, 1)
        self.minLabel_4 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_4.setObjectName("minLabel_4")
        self.gridLayout_2.addWidget(self.minLabel_4, 7, 2, 1, 1)
        self.maxSpinBox_8 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_8.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_8.setObjectName("maxSpinBox_8")
        self.gridLayout_2.addWidget(self.maxSpinBox_8, 11, 5, 1, 1)
        self.maxLabel_5 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_5.setObjectName("maxLabel_5")
        self.gridLayout_2.addWidget(self.maxLabel_5, 8, 4, 1, 1)
        self.maxSpinBox_6 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_6.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_6.setObjectName("maxSpinBox_6")
        self.gridLayout_2.addWidget(self.maxSpinBox_6, 9, 5, 1, 1)
        self.maxSpinBox_7 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_7.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_7.setObjectName("maxSpinBox_7")
        self.gridLayout_2.addWidget(self.maxSpinBox_7, 10, 5, 1, 1)
        self.referencePredictionLabel = QtWidgets.QLabel(self.groupBox)
        self.referencePredictionLabel.setObjectName("referencePredictionLabel")
        self.gridLayout_2.addWidget(self.referencePredictionLabel, 1, 0, 1, 1)
        self.minLabel_3 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_3.setObjectName("minLabel_3")
        self.gridLayout_2.addWidget(self.minLabel_3, 6, 2, 1, 1)
        self.maxLabel_6 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_6.setObjectName("maxLabel_6")
        self.gridLayout_2.addWidget(self.maxLabel_6, 9, 4, 1, 1)
        self.lowPredictionMaxLabel = QtWidgets.QLabel(self.groupBox)
        self.lowPredictionMaxLabel.setMaximumSize(QtCore.QSize(30, 16777215))
        self.lowPredictionMaxLabel.setObjectName("lowPredictionMaxLabel")
        self.gridLayout_2.addWidget(self.lowPredictionMaxLabel, 3, 2, 1, 1)
        self.minLabel_9 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_9.setObjectName("minLabel_9")
        self.gridLayout_2.addWidget(self.minLabel_9, 12, 2, 1, 1)
        self.maxSpinBox_10 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_10.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_10.setObjectName("maxSpinBox_10")
        self.gridLayout_2.addWidget(self.maxSpinBox_10, 13, 5, 1, 1)
        self.minSpinBox_12 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_12.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_12.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_12.setObjectName("minSpinBox_12")
        self.gridLayout_2.addWidget(self.minSpinBox_12, 15, 3, 1, 1)
        self.PredictionComboBox_9 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_9.setObjectName("PredictionComboBox_9")
        self.gridLayout_2.addWidget(self.PredictionComboBox_9, 12, 1, 1, 1)
        self.highPredictionMinSpinBox = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.highPredictionMinSpinBox.setMinimumSize(QtCore.QSize(130, 0))
        self.highPredictionMinSpinBox.setMaximumSize(QtCore.QSize(70, 16777215))
        self.highPredictionMinSpinBox.setObjectName("highPredictionMinSpinBox")
        self.gridLayout_2.addWidget(self.highPredictionMinSpinBox, 19, 3, 1, 1)
        self.PredictionComboBox_11 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_11.setObjectName("PredictionComboBox_11")
        self.gridLayout_2.addWidget(self.PredictionComboBox_11, 14, 1, 1, 1)
        self.PredictionComboBox_4 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_4.setObjectName("PredictionComboBox_4")
        self.gridLayout_2.addWidget(self.PredictionComboBox_4, 7, 1, 1, 1)
        self.minSpinBox_7 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_7.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_7.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_7.setObjectName("minSpinBox_7")
        self.gridLayout_2.addWidget(self.minSpinBox_7, 10, 3, 1, 1)
        self.PredictionComboBox_5 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_5.setObjectName("PredictionComboBox_5")
        self.gridLayout_2.addWidget(self.PredictionComboBox_5, 8, 1, 1, 1)
        self.maxSpinBox_3 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_3.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_3.setObjectName("maxSpinBox_3")
        self.gridLayout_2.addWidget(self.maxSpinBox_3, 6, 5, 1, 1)
        self.minSpinBox_6 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_6.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_6.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_6.setObjectName("minSpinBox_6")
        self.gridLayout_2.addWidget(self.minSpinBox_6, 9, 3, 1, 1)
        self.minSpinBox_8 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_8.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_8.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_8.setObjectName("minSpinBox_8")
        self.gridLayout_2.addWidget(self.minSpinBox_8, 11, 3, 1, 1)
        self.maxLabel_11 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_11.setObjectName("maxLabel_11")
        self.gridLayout_2.addWidget(self.maxLabel_11, 14, 4, 1, 1)
        self.maxLabel_3 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_3.setObjectName("maxLabel_3")
        self.gridLayout_2.addWidget(self.maxLabel_3, 6, 4, 1, 1)
        self.highPredictionLabel = QtWidgets.QLabel(self.groupBox)
        self.highPredictionLabel.setObjectName("highPredictionLabel")
        self.gridLayout_2.addWidget(self.highPredictionLabel, 19, 0, 1, 1)
        self.PredictionComboBox_13 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_13.setObjectName("PredictionComboBox_13")
        self.gridLayout_2.addWidget(self.PredictionComboBox_13, 16, 1, 1, 1)
        self.minSpinBox_11 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_11.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_11.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_11.setObjectName("minSpinBox_11")
        self.gridLayout_2.addWidget(self.minSpinBox_11, 14, 3, 1, 1)
        self.minLabel_2 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_2.setObjectName("minLabel_2")
        self.gridLayout_2.addWidget(self.minLabel_2, 5, 2, 1, 1)
        self.minSpinBox_3 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_3.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_3.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_3.setObjectName("minSpinBox_3")
        self.gridLayout_2.addWidget(self.minSpinBox_3, 6, 3, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.addSubPushButton = QtWidgets.QPushButton(self.groupBox)
        self.addSubPushButton.setObjectName("addSubPushButton")
        self.horizontalLayout.addWidget(self.addSubPushButton)
        self.deleteSubPushButton = QtWidgets.QPushButton(self.groupBox)
        self.deleteSubPushButton.setObjectName("deleteSubPushButton")
        self.horizontalLayout.addWidget(self.deleteSubPushButton)
        self.optimizeSubRangesCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.optimizeSubRangesCheckBox.setObjectName("optimizeSubRangesCheckBox")
        self.horizontalLayout.addWidget(self.optimizeSubRangesCheckBox)
        self.gridLayout_2.addLayout(self.horizontalLayout, 20, 0, 1, 6)
        self.PredictionComboBox_14 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_14.setObjectName("PredictionComboBox_14")
        self.gridLayout_2.addWidget(self.PredictionComboBox_14, 17, 1, 1, 1)
        self.minLabel_14 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_14.setObjectName("minLabel_14")
        self.gridLayout_2.addWidget(self.minLabel_14, 17, 2, 1, 1)
        self.minLabel_11 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_11.setObjectName("minLabel_11")
        self.gridLayout_2.addWidget(self.minLabel_11, 14, 2, 1, 1)
        self.maxSpinBox_4 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_4.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_4.setObjectName("maxSpinBox_4")
        self.gridLayout_2.addWidget(self.maxSpinBox_4, 7, 5, 1, 1)
        self.lowPredictionComboBox = QtWidgets.QComboBox(self.groupBox)
        self.lowPredictionComboBox.setObjectName("lowPredictionComboBox")
        self.gridLayout_2.addWidget(self.lowPredictionComboBox, 3, 1, 1, 1)
        self.maxLabel_9 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_9.setObjectName("maxLabel_9")
        self.gridLayout_2.addWidget(self.maxLabel_9, 12, 4, 1, 1)
        self.highPredictionMinLabel = QtWidgets.QLabel(self.groupBox)
        self.highPredictionMinLabel.setObjectName("highPredictionMinLabel")
        self.gridLayout_2.addWidget(self.highPredictionMinLabel, 19, 2, 1, 1)
        self.minLabel_13 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_13.setObjectName("minLabel_13")
        self.gridLayout_2.addWidget(self.minLabel_13, 16, 2, 1, 1)
        self.lowPredictionMaxSpinBox = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.lowPredictionMaxSpinBox.setMinimumSize(QtCore.QSize(130, 0))
        self.lowPredictionMaxSpinBox.setMaximumSize(QtCore.QSize(910, 16777215))
        self.lowPredictionMaxSpinBox.setObjectName("lowPredictionMaxSpinBox")
        self.gridLayout_2.addWidget(self.lowPredictionMaxSpinBox, 3, 3, 1, 1)
        self.minLabel_15 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_15.setObjectName("minLabel_15")
        self.gridLayout_2.addWidget(self.minLabel_15, 18, 2, 1, 1)
        self.minLabel_5 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_5.setObjectName("minLabel_5")
        self.gridLayout_2.addWidget(self.minLabel_5, 8, 2, 1, 1)
        self.PredictionComboBox_10 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_10.setObjectName("PredictionComboBox_10")
        self.gridLayout_2.addWidget(self.PredictionComboBox_10, 13, 1, 1, 1)
        self.minSpinBox_4 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_4.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_4.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_4.setObjectName("minSpinBox_4")
        self.gridLayout_2.addWidget(self.minSpinBox_4, 7, 3, 1, 1)
        self.PredictionComboBox_3 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_3.setObjectName("PredictionComboBox_3")
        self.gridLayout_2.addWidget(self.PredictionComboBox_3, 6, 1, 1, 1)
        self.maxLabel_4 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_4.setObjectName("maxLabel_4")
        self.gridLayout_2.addWidget(self.maxLabel_4, 7, 4, 1, 1)
        self.minSpinBox_5 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_5.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_5.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_5.setObjectName("minSpinBox_5")
        self.gridLayout_2.addWidget(self.minSpinBox_5, 8, 3, 1, 1)
        self.PredictionComboBox_6 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_6.setObjectName("PredictionComboBox_6")
        self.gridLayout_2.addWidget(self.PredictionComboBox_6, 9, 1, 1, 1)
        self.lowPredictionLabel = QtWidgets.QLabel(self.groupBox)
        self.lowPredictionLabel.setObjectName("lowPredictionLabel")
        self.gridLayout_2.addWidget(self.lowPredictionLabel, 3, 0, 1, 1)
        self.chooseDatalabel = QtWidgets.QLabel(self.groupBox)
        self.chooseDatalabel.setObjectName("chooseDatalabel")
        self.gridLayout_2.addWidget(self.chooseDatalabel, 0, 0, 1, 1)
        self.maxLabel_13 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_13.setObjectName("maxLabel_13")
        self.gridLayout_2.addWidget(self.maxLabel_13, 16, 4, 1, 1)
        self.minSpinBox_13 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_13.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_13.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_13.setObjectName("minSpinBox_13")
        self.gridLayout_2.addWidget(self.minSpinBox_13, 16, 3, 1, 1)
        self.maxLabel_12 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_12.setObjectName("maxLabel_12")
        self.gridLayout_2.addWidget(self.maxLabel_12, 15, 4, 1, 1)
        self.minSpinBox_10 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_10.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_10.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_10.setObjectName("minSpinBox_10")
        self.gridLayout_2.addWidget(self.minSpinBox_10, 13, 3, 1, 1)
        self.chooseDataComboBox = QtWidgets.QComboBox(self.groupBox)
        self.chooseDataComboBox.setObjectName("chooseDataComboBox")
        self.gridLayout_2.addWidget(self.chooseDataComboBox, 0, 1, 1, 1)
        self.maxSpinBox_14 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_14.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_14.setObjectName("maxSpinBox_14")
        self.gridLayout_2.addWidget(self.maxSpinBox_14, 17, 5, 1, 1)
        self.minSpinBox_14 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_14.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_14.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_14.setObjectName("minSpinBox_14")
        self.gridLayout_2.addWidget(self.minSpinBox_14, 17, 3, 1, 1)
        self.minLabel_12 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_12.setObjectName("minLabel_12")
        self.gridLayout_2.addWidget(self.minLabel_12, 15, 2, 1, 1)
        self.minSpinBox_15 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_15.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_15.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_15.setObjectName("minSpinBox_15")
        self.gridLayout_2.addWidget(self.minSpinBox_15, 18, 3, 1, 1)
        self.maxSpinBox_13 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_13.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_13.setObjectName("maxSpinBox_13")
        self.gridLayout_2.addWidget(self.maxSpinBox_13, 16, 5, 1, 1)
        self.maxLabel_10 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_10.setObjectName("maxLabel_10")
        self.gridLayout_2.addWidget(self.maxLabel_10, 13, 4, 1, 1)
        self.maxSpinBox_11 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_11.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_11.setObjectName("maxSpinBox_11")
        self.gridLayout_2.addWidget(self.maxSpinBox_11, 14, 5, 1, 1)
        self.maxLabel_15 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_15.setObjectName("maxLabel_15")
        self.gridLayout_2.addWidget(self.maxLabel_15, 18, 4, 1, 1)
        self.minSpinBox_9 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_9.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_9.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_9.setObjectName("minSpinBox_9")
        self.gridLayout_2.addWidget(self.minSpinBox_9, 12, 3, 1, 1)
        self.maxLabel_8 = QtWidgets.QLabel(self.groupBox)
        self.maxLabel_8.setObjectName("maxLabel_8")
        self.gridLayout_2.addWidget(self.maxLabel_8, 11, 4, 1, 1)
        self.minLabel_10 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_10.setObjectName("minLabel_10")
        self.gridLayout_2.addWidget(self.minLabel_10, 13, 2, 1, 1)
        self.minSpinBox_2 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.minSpinBox_2.setMinimumSize(QtCore.QSize(130, 0))
        self.minSpinBox_2.setMaximumSize(QtCore.QSize(70, 16777215))
        self.minSpinBox_2.setObjectName("minSpinBox_2")
        self.gridLayout_2.addWidget(self.minSpinBox_2, 5, 3, 1, 1)
        self.maxSpinBox = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox.setObjectName("maxSpinBox")
        self.gridLayout_2.addWidget(self.maxSpinBox, 4, 5, 1, 1)
        self.minLabel_7 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_7.setObjectName("minLabel_7")
        self.gridLayout_2.addWidget(self.minLabel_7, 10, 2, 1, 1)
        self.PredictionComboBox_7 = QtWidgets.QComboBox(self.groupBox)
        self.PredictionComboBox_7.setObjectName("PredictionComboBox_7")
        self.gridLayout_2.addWidget(self.PredictionComboBox_7, 10, 1, 1, 1)
        self.maxLabel = QtWidgets.QLabel(self.groupBox)
        self.maxLabel.setMaximumSize(QtCore.QSize(30, 16777215))
        self.maxLabel.setObjectName("maxLabel")
        self.gridLayout_2.addWidget(self.maxLabel, 4, 4, 1, 1)
        self.minLabel_8 = QtWidgets.QLabel(self.groupBox)
        self.minLabel_8.setObjectName("minLabel_8")
        self.gridLayout_2.addWidget(self.minLabel_8, 11, 2, 1, 1)
        self.maxSpinBox_2 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_2.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_2.setObjectName("maxSpinBox_2")
        self.gridLayout_2.addWidget(self.maxSpinBox_2, 5, 5, 1, 1)
        self.formLayout_3 = QtWidgets.QFormLayout()
        self.formLayout_3.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout_3.setFormAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.formLayout_3.setObjectName("formLayout_3")
        self.optimizeSubRangesLabel = QtWidgets.QLabel(self.groupBox)
        self.optimizeSubRangesLabel.setObjectName("optimizeSubRangesLabel")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.optimizeSubRangesLabel)
        self.optimizeSubRangesComboBox = QtWidgets.QComboBox(self.groupBox)
        self.optimizeSubRangesComboBox.setObjectName("optimizeSubRangesComboBox")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.optimizeSubRangesComboBox)
        self.gridLayout_2.addLayout(self.formLayout_3, 21, 0, 1, 6)
        self.maxSpinBox_9 = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.maxSpinBox_9.setMinimumSize(QtCore.QSize(130, 0))
        self.maxSpinBox_9.setObjectName("maxSpinBox_9")
        self.gridLayout_2.addWidget(self.maxSpinBox_9, 12, 5, 1, 1)
        self.index_spin = QtWidgets.QSpinBox(self.groupBox)
        self.index_spin.setObjectName("index_spin")
        self.gridLayout_2.addWidget(self.index_spin, 2, 1, 1, 1)
        self.gridLayout.addWidget(self.groupBox, 0, 0, 1, 1)

        self.retranslateUi(Form)
        self.optimizeSubRangesCheckBox.toggled['bool'].connect(self.optimizeSubRangesComboBox.setVisible)
        self.optimizeSubRangesCheckBox.toggled['bool'].connect(self.optimizeSubRangesLabel.setVisible)
        QtCore.QMetaObject.connectSlotsByName(Form)
        Form.setTabOrder(self.referencePredictionComboBox, self.lowPredictionComboBox)
        Form.setTabOrder(self.lowPredictionComboBox, self.lowPredictionMaxSpinBox)
        Form.setTabOrder(self.lowPredictionMaxSpinBox, self.PredictionComboBox)
        Form.setTabOrder(self.PredictionComboBox, self.minSpinBox)
        Form.setTabOrder(self.minSpinBox, self.maxSpinBox)
        Form.setTabOrder(self.maxSpinBox, self.PredictionComboBox_2)
        Form.setTabOrder(self.PredictionComboBox_2, self.minSpinBox_2)
        Form.setTabOrder(self.minSpinBox_2, self.maxSpinBox_2)
        Form.setTabOrder(self.maxSpinBox_2, self.PredictionComboBox_3)
        Form.setTabOrder(self.PredictionComboBox_3, self.minSpinBox_3)
        Form.setTabOrder(self.minSpinBox_3, self.maxSpinBox_3)
        Form.setTabOrder(self.maxSpinBox_3, self.PredictionComboBox_4)
        Form.setTabOrder(self.PredictionComboBox_4, self.minSpinBox_4)
        Form.setTabOrder(self.minSpinBox_4, self.maxSpinBox_4)
        Form.setTabOrder(self.maxSpinBox_4, self.PredictionComboBox_5)
        Form.setTabOrder(self.PredictionComboBox_5, self.minSpinBox_5)
        Form.setTabOrder(self.minSpinBox_5, self.maxSpinBox_5)
        Form.setTabOrder(self.maxSpinBox_5, self.PredictionComboBox_6)
        Form.setTabOrder(self.PredictionComboBox_6, self.minSpinBox_6)
        Form.setTabOrder(self.minSpinBox_6, self.maxSpinBox_6)
        Form.setTabOrder(self.maxSpinBox_6, self.PredictionComboBox_7)
        Form.setTabOrder(self.PredictionComboBox_7, self.minSpinBox_7)
        Form.setTabOrder(self.minSpinBox_7, self.maxSpinBox_7)
        Form.setTabOrder(self.maxSpinBox_7, self.PredictionComboBox_8)
        Form.setTabOrder(self.PredictionComboBox_8, self.minSpinBox_8)
        Form.setTabOrder(self.minSpinBox_8, self.maxSpinBox_8)
        Form.setTabOrder(self.maxSpinBox_8, self.PredictionComboBox_9)
        Form.setTabOrder(self.PredictionComboBox_9, self.minSpinBox_9)
        Form.setTabOrder(self.minSpinBox_9, self.maxSpinBox_9)
        Form.setTabOrder(self.maxSpinBox_9, self.PredictionComboBox_10)
        Form.setTabOrder(self.PredictionComboBox_10, self.minSpinBox_10)
        Form.setTabOrder(self.minSpinBox_10, self.maxSpinBox_10)
        Form.setTabOrder(self.maxSpinBox_10, self.PredictionComboBox_11)
        Form.setTabOrder(self.PredictionComboBox_11, self.minSpinBox_11)
        Form.setTabOrder(self.minSpinBox_11, self.maxSpinBox_11)
        Form.setTabOrder(self.maxSpinBox_11, self.PredictionComboBox_12)
        Form.setTabOrder(self.PredictionComboBox_12, self.minSpinBox_12)
        Form.setTabOrder(self.minSpinBox_12, self.maxSpinBox_12)
        Form.setTabOrder(self.maxSpinBox_12, self.PredictionComboBox_13)
        Form.setTabOrder(self.PredictionComboBox_13, self.minSpinBox_13)
        Form.setTabOrder(self.minSpinBox_13, self.maxSpinBox_13)
        Form.setTabOrder(self.maxSpinBox_13, self.PredictionComboBox_14)
        Form.setTabOrder(self.PredictionComboBox_14, self.minSpinBox_14)
        Form.setTabOrder(self.minSpinBox_14, self.maxSpinBox_14)
        Form.setTabOrder(self.maxSpinBox_14, self.PredictionComboBox_15)
        Form.setTabOrder(self.PredictionComboBox_15, self.minSpinBox_15)
        Form.setTabOrder(self.minSpinBox_15, self.maxSpinBox_15)
        Form.setTabOrder(self.maxSpinBox_15, self.highPredictionComboBox)
        Form.setTabOrder(self.highPredictionComboBox, self.highPredictionMinSpinBox)
        Form.setTabOrder(self.highPredictionMinSpinBox, self.addSubPushButton)
        Form.setTabOrder(self.addSubPushButton, self.deleteSubPushButton)
        Form.setTabOrder(self.deleteSubPushButton, self.optimizeSubRangesCheckBox)
        Form.setTabOrder(self.optimizeSubRangesCheckBox, self.optimizeSubRangesComboBox)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(("Form"))
        self.groupBox.setTitle(("Blend Submodel Predictions"))
        self.maxLabel_7.setText(("Max"))
        self.maxLabel_14.setText(("Max"))
        self.maxLabel_2.setText(("Max"))
        self.minLabel_6.setText(("Min"))
        self.minLabel.setText(("Min"))
        self.minLabel_4.setText(("Min"))
        self.maxLabel_5.setText(("Max"))
        self.referencePredictionLabel.setText(("Choose reference model prediction: "))
        self.minLabel_3.setText(("Min"))
        self.maxLabel_6.setText(("Max"))
        self.lowPredictionMaxLabel.setText(("Max"))
        self.minLabel_9.setText(("Min"))
        self.maxLabel_11.setText(("Max"))
        self.maxLabel_3.setText(("Max"))
        self.highPredictionLabel.setText(("High Model Prediction"))
        self.minLabel_2.setText(("Min"))
        self.addSubPushButton.setText(("Add submodel prediction"))
        self.deleteSubPushButton.setText(("Delete submodel prediction"))
        self.optimizeSubRangesCheckBox.setText(("Optimize "))
        self.minLabel_14.setText(("Min"))
        self.minLabel_11.setText(("Min"))
        self.maxLabel_9.setText(("Max"))
        self.highPredictionMinLabel.setText(("Min"))
        self.minLabel_13.setText(("Min"))
        self.minLabel_15.setText(("Min"))
        self.minLabel_5.setText(("Min"))
        self.maxLabel_4.setText(("Max"))
        self.lowPredictionLabel.setText(("Low Model Prediction"))
        self.chooseDatalabel.setText(("Choose data"))
        self.maxLabel_13.setText(("Max"))
        self.maxLabel_12.setText(("Max"))
        self.minLabel_12.setText(("Min"))
        self.maxLabel_10.setText(("Max"))
        self.maxLabel_15.setText(("Max"))
        self.maxLabel_8.setText(("Max"))
        self.minLabel_10.setText(("Min"))
        self.minLabel_7.setText(("Min"))
        self.maxLabel.setText(("Max"))
        self.minLabel_8.setText(("Min"))
        self.optimizeSubRangesLabel.setText(("Choose reference to optimize blending ranges:"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())




from PyQt5 import QtWidgets
from sklearn.linear_model.least_angle import Lars
from sklearn.linear_model.least_angle import LarsCV

from point_spectra_gui.ui.LARS import Ui_Form
from point_spectra_gui.util.Modules import Modules


class Ui_Form(Ui_Form, Modules):
    def setupUi(self, Form):
        super().setupUi(Form)
        self.checkMinAndMax()
        self.connectWidgets()

    def get_widget(self):
        return self.formGroupBox

    def setHidden(self, bool):
        self.get_widget().setHidden(bool)

    def connectWidgets(self):
        # LARS/         # LARSCV

        lars = Lars()
        self.fit_interceptCheckBox.setChecked(lars.fit_intercept)
        self.normalizeCheckBox.setChecked(lars.normalize)
        self.n_nonzero_coefsSpinBox.setValue(lars.n_nonzero_coefs)
        self.positiveCheckBox.setChecked(lars.positive)

    def run(self):
        params = {
            'fit_intercept': self.fit_interceptCheckBox.isChecked(),
            'verbose': False,
            'normalize': self.normalizeCheckBox.isChecked(),
            'precompute': 'auto',
            'n_nonzero_coefs': self.n_nonzero_coefsSpinBox.value(),
            'copy_X': True,
            'fit_path': True,
            'positive': self.positiveCheckBox.isChecked()
            }
        return params, self.getChangedValues(params, Lars())

if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



def del_layout(qlayout, idx):
    """
    This method will delete the n - idxed layout that exists in qlayout

    +--------------------+
    | +----------------+ | The outer box is QLayout
    | |       0        | | The inner Layouts marked as 0 and 1 can be indexed by qlayout
    | |                | | You can reference the index of these inner layouts by using
    | +----------------+ | the method takeAt()
    | +----------------+ | In this case if I want to access Layout 2
    | |       1        | | I use qlayout.takeAt(1)
    | |                | |
    | +----------------+ |
    +--------------------+


    :param qlayout:
    :param idx:
    :return:
    """
    to_delete = qlayout.takeAt(qlayout.count() - idx)  # remove the layout item at n-1 index
    if to_delete is not None:  # We run this method as long as there are objects
        while to_delete.count():  # while the count is not 0
            item = to_delete.takeAt(0)  # grab the layout item at 0th index
            widget = item.widget()  # get the widget at this location
            if widget is not None:  # if there is an object in this widget
                widget.deleteLater()  # delete this widget
            else:
                pass


def del_qwidget(QWidget):
    QWidget.close()
    QWidget.deleteLater()



from PyQt5 import QtWidgets
from libpysat.transform.baseline_code.polyfit import PolyFit

from point_spectra_gui.ui.Polyfit import Ui_Form
from point_spectra_gui.util.Modules import Modules


class Ui_Form(Ui_Form, Modules):
    def setupUi(self, Form):
        super().setupUi(Form)
        self.checkMinAndMax()
        Modules.setupUi(self, Form)

    def get_widget(self):
        return self.groupbox

    def setHidden(self, bool):
        self.get_widget().setHidden(bool)

    def connectWidgets(self):
        br = PolyFit()
        self.orderSpinBox.setValue(br.poly_order_)
        self.numOfStandardDeviationsSpinBox.setValue(br.stdv_)

    def run(self):
        methodParameters = {'poly_order_': self.orderSpinBox.value(),
                            'stdv_': self.numOfStandardDeviationsSpinBox.value(),
                            'max_iter_': self.maxNumOfIterationsSpinBox.value()}
        return methodParameters, self.getChangedValues(methodParameters, PolyFit())


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



from PyQt5 import QtWidgets

from point_spectra_gui.util import Qtickle
from point_spectra_gui.core.dimensionalityReductionMethods import *
from point_spectra_gui.ui.DimensionalityReduction import Ui_Form
from point_spectra_gui.util.Modules import Modules
from point_spectra_gui.util.spectral_data import spectral_data
from libpysat.transform.dim_red import dim_red

class DimensionalityReduction(Ui_Form, Modules):
    def setupUi(self, Form):
        self.Form = Form
        super().setupUi(Form)
        Modules.setupUi(self, Form)
        self.dimensionalityReductionMethods()

    def get_widget(self):
        return self.formGroupBox

    def connectWidgets(self):
        self.algorithm_list = ['Choose an algorithm',
                               'PCA',
                               'FastICA',
                               'JADE-ICA',
                               't-SNE',
                               'LLE']

        self.setComboBox(self.chooseDataComboBox, self.datakeys)
        self.setComboBox(self.chooseMethodComboBox, self.algorithm_list)
        self.chooseMethodComboBox.currentIndexChanged.connect(
            lambda: self.make_dimred_widget(self.chooseMethodComboBox.currentText()))

    def getGuiParams(self):
        """
        Overriding Modules' getGuiParams, because I'll need to do a list of lists
        in order to obtain the regressionMethods' parameters
        """
        self.qt = Qtickle.Qtickle(self)
        s = []
        s.append(self.qt.guiSave())
        for items in self.alg:
            s.append(items.getGuiParams())
        return s

    def setGuiParams(self, dict):
        """
        Overriding Modules' setGuiParams, because we are accessing a list of lists
        And each submodule contains it's own `setGuiParams`
        """
        self.qt = Qtickle.Qtickle(self)
        self.qt.guiRestore(dict[0])
        for i in range(len(dict)):
            self.alg[i - 1].setGuiParams(dict[i])

    def selectiveSetGuiParams(self, dict):
        """
        Override Modules' selective Restore function

        Setup Qtickle
        selectively restore the UI, the data to do that will be in the 0th element of the dictionary
        We will then iterate through the rest of the dictionary
        Will now restore the parameters for the algorithms in the list, Each of the algs have their own selectiveSetGuiParams

        :param dict:
        :return:
        """

        self.qt = Qtickle.Qtickle(self)
        self.qt.selectiveGuiRestore(dict[0])
        for i in range(len(dict)):
            self.alg[i - 1].selectiveSetGuiParams(dict[i])

    def setup(self):
        method = self.chooseMethodComboBox.currentText()
        datakey = self.chooseDataComboBox.currentText()
        params, modelkey = self.getMethodParams(self.chooseMethodComboBox.currentIndex())
        if method != 'Choose an algorithm':
            try:
                for i in list(range(1, params['n_components'])):  # will need to revisit this for other methods that don't use n_components to make sure column names still mamke sense
                    self.data[datakey].df[(method, str(i))] = 99999 #fill with dummy data to be replaced when actually run
            except:
                pass


    def run(self):
        method = self.chooseMethodComboBox.currentText()
        datakey = self.chooseDataComboBox.currentText()
        # xvars = [str(x.text()) for x in self.xVariableList.selectedItems()]
        params, modelkey = self.getMethodParams(self.chooseMethodComboBox.currentIndex())
        load_fit = False
        col = 'wvl'
        df, do_dim_red=dim_red(self.data[datakey].df,col, method, [], params, load_fit=load_fit)
        self.data[datakey] = spectral_data(df, dim_red = do_dim_red)

    def make_dimred_widget(self, alg, params=None):
        self.hideAll()
        #print(alg)
        for i in range(len(self.algorithm_list)):
            if alg == self.algorithm_list[i]:
                self.alg[i - 1].setHidden(False)

    def hideAll(self):
        for a in self.alg:
            a.setHidden(True)

    def dimensionalityReductionMethods(self):
        self.alg = []
        list_forms = [dimred_PCA,
                      dimred_FastICA,
                      dimred_JADE,
                      dimred_tSNE,
                      dimred_LLE]
        for items in list_forms:
            self.alg.append(items.Ui_Form())
            self.alg[-1].setupUi(self.Form)
            self.dim_reduction_vlayout.addWidget(self.alg[-1].get_widget())
            self.alg[-1].setHidden(True)

    def getMethodParams(self, index):
        return self.alg[index - 1].run()


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)

    Form = QtWidgets.QWidget()
    ui = DimensionalityReduction()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



from PyQt5 import QtWidgets
from sklearn.linear_model import ElasticNet
from sklearn.linear_model.coordinate_descent import ElasticNetCV

from point_spectra_gui.ui.ElasticNet import Ui_Form
from point_spectra_gui.util.Modules import Modules


class Ui_Form(Ui_Form, ElasticNet, ElasticNetCV, Modules):
    def setupUi(self, Form):
        super().setupUi(Form)
        self.checkMinAndMax()
        self.connectWidgets()

    def get_widget(self):
        return self.groupBox

    def setHidden(self, bool):
        self.get_widget().setHidden(bool)

    def connectWidgets(self):
        en = ElasticNet()

        self.alpha_text.setText(str(en.alpha))
        self.enl1_ratioDoubleSpinBox.setValue(en.l1_ratio)
        self.enfit_interceptCheckBox.setChecked(en.fit_intercept)
        self.enmax_iterSpinBox.setValue(en.max_iter)
        self.entolDoubleSpinBox.setValue(en.tol)
        self.enpositiveCheckBox.setChecked(en.positive)


    def run(self):
        params = {
                'alpha': float(self.alpha_text.text()),
                'l1_ratio': self.enl1_ratioDoubleSpinBox.value(),
                'fit_intercept': self.enfit_interceptCheckBox.isChecked(),
                'precompute': True,
                'max_iter': self.enmax_iterSpinBox.value(),
                'copy_X': True,
                'tol': self.entolDoubleSpinBox.value(),
                'warm_start': True,
                'positive': self.enpositiveCheckBox.isChecked(),
                'selection': self.enselectionComboBox.currentText()
                }
        return params, self.getChangedValues(params, ElasticNet())


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())
